@@ -1,23 +1,23 @@
-# -*- coding: utf-8 -*-
+# -*- coding: utf-8; -*-
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2016 Lance Edgar
+#  Copyright © 2010-2017 Lance Edgar
 #
 #  This file is part of Rattail.
 #
 #  Rattail is free software: you can redistribute it and/or modify it under the
-#  terms of the GNU Affero General Public License as published by the Free
-#  Software Foundation, either version 3 of the License, or (at your option)
-#  any later version.
+#  terms of the GNU General Public License as published by the Free Software
+#  Foundation, either version 3 of the License, or (at your option) any later
+#  version.
 #
 #  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
 #  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
-#  more details.
+#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+#  details.
 #
-#  You should have received a copy of the GNU Affero General Public License
-#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
+#  You should have received a copy of the GNU General Public License along with
+#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
 #
 ################################################################################
 """
@@ -38,10 +38,14 @@ import warnings
 import logging
 from getpass import getpass
 
+import six
+
 from rattail import __version__
+from rattail.core import Object
 from rattail.util import load_entry_points, load_object
 from rattail.console import Progress
 from rattail.config import make_config
+from rattail.util import progress_loop
 
 
 log = logging.getLogger(__name__)
@@ -78,7 +82,8 @@ def date_argument(string):
     return date
 
 
-class Command(object):
+@six.python_2_unicode_compatible
+class Command(Object):
     """
     The primary command for the application.
 
@@ -90,7 +95,7 @@ class Command(object):
     """
     name = 'rattail'
     version = __version__
-    description = "Retail Software Framework"
+    description = "Rattail Software Framework"
     long_description = """
 Rattail is a retail software framework.
 
@@ -104,11 +109,12 @@ See the file COPYING.txt for more information.
     stdout = sys.stdout
     stderr = sys.stderr
 
-    def __init__(self):
+    def __init__(self, **kwargs):
+        super(Command, self).__init__(**kwargs)
         self.subcommands = load_entry_points('{}.commands'.format(self.name.replace('-', '_')))
 
-    def __unicode__(self):
-        return unicode(self.name)
+    def __str__(self):
+        return self.name
 
     @property
     def db_config_section(self):
@@ -156,18 +162,19 @@ See the file COPYING.txt for more information.
 
         The output will include a list of available subcommands.
         """
-        self.stdout.write("""{0}
+        # TODO: this should leverage parser config...
+        self.stdout.write("""{0.description}
 
-Usage: {1} [options] <command> [command-options]
+Usage: {0.name} [options] <command> [command-options]
 
 Options:
   -c PATH, --config=PATH
                     Config path (may be specified more than once)
-  -n, --no-init     Don't load config before executing command
   -P, --progress    Show progress indicators (where relevant)
+  -R, --runas       Optional username to impersonate when running command
   -V, --version     Display program version and exit
 
-Commands:\n""".format(self.description, self.name))
+Commands:\n""".format(self))
 
         for cmd in self.iter_subcommands():
             self.stdout.write("  {0:<16s}  {1}\n".format(cmd.name, cmd.description))
@@ -186,9 +193,15 @@ Commands:\n""".format(self.description, self.name))
 
         parser.add_argument('-c', '--config', action='append', dest='config_paths',
                             metavar='PATH')
+
+        # TODO: i think these aren't really being used in practice..?
         parser.add_argument('-n', '--no-init', action='store_true', default=False)
         parser.add_argument('--no-extend-config', dest='extend_config', action='store_false')
+
         parser.add_argument('-P', '--progress', action='store_true', default=False)
+        parser.add_argument('--runas', '-R', metavar='USERNAME',
+                            help="Optional username to impersonate when running the command.  "
+                            "This is only relevant for / used by certain commands.")
         parser.add_argument('--stdout', metavar='PATH', type=argparse.FileType('w'),
                             help="Optional path to which STDOUT should be effectively redirected.")
         parser.add_argument('--stderr', metavar='PATH', type=argparse.FileType('w'),
@@ -238,6 +251,9 @@ Commands:\n""".format(self.description, self.name))
         cmd = self.subcommands[cmd](self, config)
         cmd.show_progress = args.progress
         cmd.progress = Progress if args.progress else None
+        # TODO: make this default to something from config?
+        cmd.runas_username = args.runas or None
+        log.debug("running '{} {}' with args: {}".format(self.name, cmd.name, args.argv))
         cmd._run(*(args.command + args.argv))
 
 
@@ -263,11 +279,32 @@ class Subcommand(object):
     def __repr__(self):
         return "Subcommand(name={0})".format(repr(self.name))
 
+    @property
+    def model(self):
+        return self.parent.db_model
+
     def add_parser_args(self, parser):
         """
         Configure additional arguments for the subcommand argument parser.
         """
         pass
+
+    def make_session(self):
+        return self.parent.db_session_factory()
+
+    def get_runas_user(self, session=None, username=None):
+        """
+        Returns a proper User object, which the app should "run as"
+        """
+        from rattail.db.util import short_session
+
+        if username is None:
+            username = self.runas_username or self.config.require('rattail', 'runas.default')
+        with short_session(session) as s:
+            return s.query(self.model.User).filter_by(username=username).one()
+
+    def progress_loop(self, func, items, factory=None, **kwargs):
+        return progress_loop(func, items, factory or self.progress, **kwargs)
             
     def _run(self, *args):
         args = self.parser.parse_args(list(args))
@@ -282,10 +319,10 @@ class Subcommand(object):
 
 class CloneDatabase(Subcommand):
     """
-    Clone data from any source database into any target database.
+    Clone (supported) data from a source DB to a target DB
     """
     name = 'clonedb'
-    description = "Clone data from a source to a target database."
+    description = __doc__.strip()
 
     def add_parser_args(self, parser):
         parser.add_argument('source_engine',
@@ -349,14 +386,10 @@ class CloneDatabase(Subcommand):
 
 class DataSync(Subcommand):
     """
-    Interacts with the datasync daemon.  This command expects a subcommand; one
-    of the following:
-
-    * ``rattail datasync start``
-    * ``rattail datasync stop``
+    Manage the data sync daemon
     """
     name = 'datasync'
-    description = "Manage the DataSync daemon"
+    description = __doc__.strip()
 
     def add_parser_args(self, parser):
         subparsers = parser.add_subparsers(title='subcommands')
@@ -370,8 +403,11 @@ class DataSync(Subcommand):
 
         parser.add_argument('-p', '--pidfile', metavar='PATH', default='/var/run/rattail/datasync.pid',
                             help="Path to PID file.")
-        parser.add_argument('-D', '--do-not-daemonize',
-                            action='store_false', dest='daemonize', default=True,
+        parser.add_argument('--daemonize', action='store_true', default=True, # TODO: should default to False
+                            help="Daemonize when starting.")
+        parser.add_argument('--no-daemonize',
+                            '-D', '--do-not-daemonize', # TODO: (re)move these?
+                            action='store_false', dest='daemonize',
                             help="Do not daemonize when starting.")
         parser.add_argument('-T', '--timeout', metavar='MINUTES', type=int, default=0,
                             help="Optional timeout (in minutes) for use with the 'wait' command.  "
@@ -453,8 +489,11 @@ class EmailBouncer(Subcommand):
 
         parser.add_argument('-p', '--pidfile', metavar='PATH', default='/var/run/rattail/bouncer.pid',
                             help="Path to PID file.")
-        parser.add_argument('-D', '--do-not-daemonize',
-                            action='store_false', dest='daemonize', default=True,
+        parser.add_argument('--daemonize', action='store_true', default=True, # TODO: should default to False
+                            help="Daemonize when starting.")
+        parser.add_argument('--no-daemonize',
+                            '-D', '--do-not-daemonize', # TODO: (re)move these?
+                            action='store_false', dest='daemonize',
                             help="Do not daemonize when starting.")
 
     def run(self, args):
@@ -475,10 +514,10 @@ class EmailBouncer(Subcommand):
 
 class DateOrganize(Subcommand):
     """
-    Organize files in a given directory, according to date.
+    Organize files in a given directory, according to date
     """
     name = 'date-organize'
-    description = "Organize files in a given directory according to date."
+    description = __doc__.strip()
 
     def add_parser_args(self, parser):
         parser.add_argument('folder', metavar='PATH',
@@ -614,7 +653,7 @@ class FileMonitorCommand(Subcommand):
     """
 
     name = 'filemon'
-    description = "Manage the file monitor service"
+    description = "Manage the file monitor daemon"
 
     def add_parser_args(self, parser):
         subparsers = parser.add_subparsers(title='subcommands')
@@ -627,8 +666,11 @@ class FileMonitorCommand(Subcommand):
         if sys.platform == 'linux2':
             parser.add_argument('-p', '--pidfile',
                                 help="Path to PID file.", metavar='PATH')
-            parser.add_argument('-D', '--do-not-daemonize',
-                                action='store_false', dest='daemonize', default=True,
+            parser.add_argument('--daemonize', action='store_true', default=True, # TODO: should default to False
+                                help="Daemonize when starting.")
+            parser.add_argument('--no-daemonize',
+                                '-D', '--do-not-daemonize', # TODO: (re)move these?
+                                action='store_false', dest='daemonize',
                                 help="Do not daemonize when starting.")
 
         elif sys.platform == 'win32': # pragma no cover
@@ -711,251 +753,6 @@ class FileMonitorCommand(Subcommand):
                 service.delayed_auto_start_service(name)
 
 
-class OldImportSubcommand(Subcommand):
-    """
-    Base class for subcommands which use the data importing system.
-    """
-    supports_versioning = True
-
-    def add_parser_args(self, parser):
-        handler = self.get_handler(quiet=True)
-        if self.supports_versioning:
-            parser.add_argument('--no-versioning', action='store_true',
-                                help="Disables versioning during the import.  This is "
-                                "intended to be useful e.g. during initial import, where "
-                                "the process can be quite slow even without the overhead "
-                                "of versioning.")
-        parser.add_argument('--warnings', '-W', action='store_true',
-                            help="Whether to log warnings if any data model "
-                            "writes occur.  Intended to help stay in sync "
-                            "with an external data source.")
-        parser.add_argument('--max-updates', type=int,
-                            help="Maximum number of record updates (or additions) which, if "
-                            "reached, should cause the importer to stop early.  Note that the "
-                            "updates which have completed will be committed unless a dry run "
-                            "is in effect.")
-        parser.add_argument('--dry-run', action='store_true',
-                            help="Go through the motions and allow logging to occur, "
-                            "but do not actually commit the transaction at the end.")
-        parser.add_argument('models', nargs='*', metavar='MODEL',
-                            help="Which models to import.  If none are specified, all models will "
-                            "be imported.  Or, specify only those you wish to import.  Supported "
-                            "models are: {0}".format(', '.join(handler.get_importer_keys())))
-
-    def run(self, args):
-        log.info("begin {0} for data model(s): {1}".format(
-                self.name, ', '.join(args.models or ["ALL"])))
-
-        Session = self.parent.db_session_factory
-        if self.supports_versioning:
-            if args.no_versioning:
-                from rattail.db.continuum  import disable_versioning
-                disable_versioning()
-            session = Session(continuum_user=self.continuum_user)
-        else:
-            session = Session()
-
-        self.import_data(args, session)
-
-        if args.dry_run:
-            session.rollback()
-            log.info("dry run, so transaction was rolled back")
-        else:
-            session.commit()
-            log.info("transaction was committed")
-        session.close()
-
-    def get_handler_factory(self, quiet=False):
-        """
-        This method must return a factory, which will in turn generate a
-        handler instance to be used by the command.  Note that you *must*
-        override this method.
-        """
-        raise NotImplementedError
-
-    def get_handler(self, **kwargs):
-        """
-        Returns a handler instance to be used by the command.
-        """
-        factory = self.get_handler_factory(quiet=kwargs.pop('quiet', False))
-        return factory(getattr(self, 'config', None), **kwargs)
-
-    @property
-    def continuum_user(self):
-        """
-        Info needed to assign the Continuum user for the database session.
-        """
-
-    def import_data(self, args, session):
-        """
-        Perform a data import, with the given arguments and database session.
-        """
-        handler = self.get_handler(session=session)
-        models = args.models or handler.get_importer_keys()
-        updates = handler.import_data(models, max_updates=args.max_updates,
-                                      progress=self.progress)
-        if args.warnings and updates:
-            handler.process_warnings(updates, command=self, models=models, dry_run=args.dry_run,
-                                     render_record=self.get_record_renderer(),
-                                     progress=self.progress)
-
-    def get_record_renderer(self):
-        """
-        Get the record renderer for email notifications.  Note that config may
-        override the default.
-        """
-        spec = self.config.get('{0}.{1}'.format(self.parent.name, self.name), 'record_renderer',
-                               default='rattail.db.importing:RecordRenderer')
-        return load_object(spec)(self.config)
-
-
-class NewImportSubcommand(Subcommand):
-    """
-    Base class for subcommands which use the (new) data importing system.
-    """
-
-    def get_handler_factory(self, args=None):
-        """
-        This method must return a factory, which will in turn generate a
-        handler instance to be used by the command.  Note that you *must*
-        override this method.
-        """
-        raise NotImplementedError
-
-    def get_handler(self, args=None, **kwargs):
-        """
-        Returns a handler instance to be used by the command.
-        """
-        factory = self.get_handler_factory(args)
-        kwargs = self.get_handler_kwargs(args, **kwargs)
-        kwargs['command'] = self
-        return factory(getattr(self, 'config', None), **kwargs)
-
-    def get_handler_kwargs(self, args, **kwargs):
-        """
-        Return a dict of kwargs to be passed to the handler factory.
-        """
-        return kwargs
-
-    def add_parser_args(self, parser):
-        handler = self.get_handler()
-
-        # model names (aka importer keys)
-        parser.add_argument('models', nargs='*', metavar='MODEL',
-                            help="Which data models to import.  If you specify any, then only data "
-                            "for those models will be imported.  If you do not specify any, then all "
-                            "*default* models will be imported.  Supported models are: ({})".format(
-                                ', '.join(handler.get_importer_keys())))
-
-        # start/end date
-        parser.add_argument('--start-date', type=date_argument,
-                            help="Optional (inclusive) starting point for date range, by which host "
-                            "data should be filtered.  Only used by certain importers.")
-        parser.add_argument('--end-date', type=date_argument,
-                            help="Optional (inclusive) ending point for date range, by which host "
-                            "data should be filtered.  Only used by certain importers.")
-
-        # allow create?
-        parser.add_argument('--create', action='store_true', default=True,
-                            help="Allow new records to be created during the import.")
-        parser.add_argument('--no-create', action='store_false', dest='create',
-                            help="Do not allow new records to be created during the import.")
-        parser.add_argument('--max-create', type=int, metavar='COUNT',
-                            help="Maximum number of records which may be created, after which a "
-                            "given import task should stop.  Note that this applies on a per-model "
-                            "basis and not overall.")
-
-        # allow update?
-        parser.add_argument('--update', action='store_true', default=True,
-                            help="Allow existing records to be updated during the import.")
-        parser.add_argument('--no-update', action='store_false', dest='update',
-                            help="Do not allow existing records to be updated during the import.")
-        parser.add_argument('--max-update', type=int, metavar='COUNT',
-                            help="Maximum number of records which may be updated, after which a "
-                            "given import task should stop.  Note that this applies on a per-model "
-                            "basis and not overall.")
-
-        # allow delete?
-        parser.add_argument('--delete', action='store_true', default=False,
-                            help="Allow records to be deleted during the import.")
-        parser.add_argument('--no-delete', action='store_false', dest='delete',
-                            help="Do not allow records to be deleted during the import.")
-        parser.add_argument('--max-delete', type=int, metavar='COUNT',
-                            help="Maximum number of records which may be deleted, after which a "
-                            "given import task should stop.  Note that this applies on a per-model "
-                            "basis and not overall.")
-
-        # max total changes, per model
-        parser.add_argument('--max-total', type=int, metavar='COUNT',
-                            help="Maximum number of *any* record changes which may occur, after which "
-                            "a given import task should stop.  Note that this applies on a per-model "
-                            "basis and not overall.")
-
-        # treat changes as warnings?
-        parser.add_argument('--warnings', '-W', action='store_true',
-                            help="Set this flag if you expect a \"clean\" import, and wish for any "
-                            "changes which do occur to be processed further and/or specially.  The "
-                            "behavior of this flag is ultimately up to the import handler, but the "
-                            "default is to send an email notification.")
-
-        # dry run?
-        parser.add_argument('--dry-run', action='store_true',
-                            help="Go through the full motions and allow logging etc. to "
-                            "occur, but rollback (abort) the transaction at the end.")
-
-    def run(self, args):
-        log.info("begin `{} {}` for data models: {}".format(
-                self.parent.name, self.name, ', '.join(args.models or ["(ALL)"])))
-
-        Session = self.parent.db_session_factory
-        session = Session()
-
-        self.import_data(args, session)
-
-        if args.dry_run:
-            session.rollback()
-            log.info("dry run, so transaction was rolled back")
-        else:
-            session.commit()
-            log.info("transaction was committed")
-        session.close()
-
-    def import_data(self, args, session):
-        """
-        Perform a data import, with the given arguments and database session.
-        """
-        handler = self.get_handler(args=args, session=session, progress=self.progress)
-        models = args.models or handler.get_default_keys()
-        log.debug("using handler: {}".format(handler))
-        log.debug("importing models: {}".format(models))
-        log.debug("args are: {}".format(args))
-        handler.import_data(models, args)
-
-
-class ImportCSV(OldImportSubcommand):
-    """
-    Import data from a CSV file.
-    """
-    name = 'import-csv'
-    description = "Import data from a CSV file."
-
-    def add_parser_args(self, parser):
-        super(ImportCSV, self).add_parser_args(parser)
-        parser.add_argument('importer',
-                            help="Spec string for importer class which should handle the import.")
-        parser.add_argument('csv_path',
-                            help="Path to the data file which will be imported.")
-
-    def import_data(self, args, session):
-        from rattail.db.importing.providers.csv import make_provider
-
-        provider = make_provider(self.config, session, args.importer, data_path=args.csv_path)
-        data = provider.get_data(progress=self.progress)
-        affected = provider.importer.import_data(data, provider.supported_fields, 'uuid',
-                                                 progress=self.progress)
-        log.info("added or updated {0} {1} records".format(affected, provider.model_name))
-
-
 class LoadHostDataCommand(Subcommand):
     """
     Loads data from the Rattail host database, if one is configured.
@@ -1043,7 +840,7 @@ class MakeConfig(Subcommand):
 
         template_path = resource_path('rattail:data/config/{}.conf'.format(args.type))
         if not os.path.exists(template_path):
-            self.stderr.write("config template not found: {}".format(template_path))
+            self.stderr.write("config template not found: {}\n".format(template_path))
             sys.exit(1)
 
         output_path = os.path.abspath(args.output)
@@ -1070,6 +867,8 @@ class MakeUser(Subcommand):
         parser.add_argument('-A', '--admin', action='store_true',
                             help="Whether the new user should have admin rights within "
                             "the system (if applicable).")
+        parser.add_argument('--password',
+                            help="Optional password to set for the new user.")
         parser.add_argument('--full-name',
                             help="Full (display) name for the new user (if applicable).")
         parser.add_argument('--comment',
@@ -1090,7 +889,9 @@ class MakeUser(Subcommand):
             args.system, args.username))
         sys.exit(1)
 
-    def obtain_password(self):
+    def obtain_password(self, args):
+        if args.password:
+            return args.password
         try:
             password = None
             while not password:
@@ -1110,9 +911,15 @@ class MakeUser(Subcommand):
             return self.user_exists(args)
 
         user = model.User(username=args.username)
-        auth.set_user_password(user, self.obtain_password())
+        auth.set_user_password(user, self.obtain_password(args))
         if args.admin:
             user.roles.append(auth.administrator_role(session))
+        if args.full_name:
+            kwargs = {'display_name': args.full_name}
+            words = args.full_name.split()
+            if len(words) == 2:
+                kwargs.update({'first_name': words[0], 'last_name': words[1]})
+            user.person = model.Person(**kwargs)
         session.add(user)
         session.commit()
         session.close()
@@ -1131,7 +938,7 @@ class MakeUser(Subcommand):
         if users.user_exists(args.username):
             return self.user_exists(args)
 
-        return users.create_user(args.username, self.obtain_password(),
+        return users.create_user(args.username, self.obtain_password(args),
                                  full_name=args.full_name, comment=args.comment)
 
 
@@ -1187,35 +994,6 @@ class PalmCommand(Subcommand):
                 sys.stderr.write(str(error) + '\n')
                 
 
-class PurgeBatchesCommand(Subcommand):
-    """
-    Purges stale batches from the database; called as:
-
-    .. code-block:: sh
-
-       rattail purge-batches
-    """
-    name = 'purge-batches'
-    description = "Purge stale batches from the database"
-
-    def add_parser_args(self, parser):
-        parser.add_argument('-A', '--all', action='store_true',
-                            help="Purge ALL batches regardless of purge date")
-        parser.add_argument('--date', '-D', type=date_argument,
-                            help="Optional effective date for the purge.  If "
-                            "none is specified, the current date is assumed.")
-
-    def run(self, args):
-        from alembic.util import obfuscate_url_pw
-        from rattail.db import Session
-        from rattail.db.batches import util
-
-        log.info("purging batches from database: {0}".format(obfuscate_url_pw(Session.kw['bind'].url)))
-        normal = util.purge_batches(effective_date=args.date, purge_everything=args.all)
-        orphaned = util.purge_orphaned_batches()
-        log.info("purged {0} normal and {1} orphaned batches".format(normal, orphaned))
-
-
 def main(*args):
     """
     The primary entry point for the Rattail command system.
