@@ -2,22 +2,22 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2016 Lance Edgar
+#  Copyright © 2010-2017 Lance Edgar
 #
 #  This file is part of Rattail.
 #
 #  Rattail is free software: you can redistribute it and/or modify it under the
-#  terms of the GNU Affero General Public License as published by the Free
-#  Software Foundation, either version 3 of the License, or (at your option)
-#  any later version.
+#  terms of the GNU General Public License as published by the Free Software
+#  Foundation, either version 3 of the License, or (at your option) any later
+#  version.
 #
 #  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
 #  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
-#  more details.
+#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+#  details.
 #
-#  You should have received a copy of the GNU Affero General Public License
-#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
+#  You should have received a copy of the GNU General Public License along with
+#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
 #
 ################################################################################
 """
@@ -27,6 +27,7 @@ Application Configuration
 from __future__ import unicode_literals, absolute_import
 
 import os
+import re
 import sys
 import shlex
 import datetime
@@ -35,7 +36,7 @@ import warnings
 import logging
 import logging.config
 
-from rattail.util import load_entry_points
+from rattail.util import load_entry_points, import_module_path
 from rattail.exceptions import WindowsExtensionsNotInstalled, ConfigurationError
 from rattail.files import temp_path
 from rattail.logging import TimeConverter
@@ -353,10 +354,23 @@ class RattailConfig(object):
 
     def production(self):
         """
-        Returns boolean indicating whether the config indicates production mode.
+        Returns boolean indicating whether the app is running in production mode
         """
         return self.getbool('rattail', 'production', default=False)
 
+    def demo(self):
+        """
+        Returns boolean indicating whether the app is running in demo mode
+        """
+        return self.getbool('rattail', 'demo', default=False)
+
+    def versioning_enabled(self):
+        """
+        Returns boolean indicating whether data versioning is enabled.
+        """
+        return self.getbool('rattail.db', 'versioning.enabled', usedb=False,
+                            default=False)
+
     def workdir(self, require=False):
         """
         Returns boolean indicating whether the config indicates production mode.
@@ -374,15 +388,41 @@ class RattailConfig(object):
             return os.path.join(path, key)
         return path
 
-    def batch_filepath(self, key, uuid, filename, makedirs=False):
+    def batch_filepath(self, key, uuid, filename=None, makedirs=False):
         """
-        Returns absolute path to batch data file, generated from the given args.
+        Returns absolute path to a batch's data folder, with optional filename
+        appended.  If ``makedirs`` is set, the batch data folder will be
+        created if it does not already exist.
         """
         rootdir = self.batch_filedir(key)
         filedir = os.path.join(rootdir, uuid[:2], uuid[2:])
         if makedirs and not os.path.exists(filedir):
             os.makedirs(filedir)
-        return os.path.join(filedir, filename)
+        if filename:
+            return os.path.join(filedir, filename)
+        return filedir
+
+    def export_filedir(self, key=None):
+        """
+        Returns path to root folder where exports (optionally of type 'key')
+        are stored.
+        """
+        path = os.path.abspath(self.require('rattail', 'export.files'))
+        if key:
+            return os.path.join(path, key)
+        return path
+
+    def export_filepath(self, key, uuid, filename=None, makedirs=False):
+        """
+        Returns absolute path to export data file, generated from the given args.
+        """
+        rootdir = self.export_filedir(key)
+        filedir = os.path.join(rootdir, uuid[:2], uuid[2:])
+        if makedirs and not os.path.exists(filedir):
+            os.makedirs(filedir)
+        if filename:
+            return os.path.join(filedir, filename)
+        return filedir
 
     def datasync_url(self):
         """
@@ -390,6 +430,26 @@ class RattailConfig(object):
         """
         return self.get('rattail.datasync', 'url')
 
+    def get_enum(self):
+        """
+        Returns a reference to configured 'enum' module; defaults to
+        :mod:`rattail.enum`.
+        """
+        spec = self.get('rattail', 'enum', default='rattail.enum')
+        return import_module_path(spec)
+
+    def get_store(self, session):
+        """
+        Returns a :class:`rattail.db.model.Store` instance corresponding to app
+        config, or ``None``.
+        """
+        from rattail.db import api
+
+        store = self.get('rattail', 'store')
+        if store:
+            return api.get_store(session, store)
+
+
     ##############################
     # deprecated methods
     ##############################
@@ -599,3 +659,29 @@ class ConfigProfile(object):
 
     def _config_list(self, option):
         return parse_list(self._config_string(option))
+
+
+class FreeTDSLoggingFilter(logging.Filter):
+    """
+    Custom logging filter, to suppress certain "write to server failed"
+    messages relating to FreeTDS database connections.  They seem harmless and
+    just cause unwanted error emails.
+    """
+
+    def __init__(self, *args, **kwargs):
+        logging.Filter.__init__(self, *args, **kwargs)
+        self.pattern = re.compile(r'(?:Read from|Write to) the server failed')
+
+    def filter(self, record):
+        if (record.name == 'sqlalchemy.pool.QueuePool'
+            and record.funcName == '_finalize_fairy'
+            and record.levelno == logging.ERROR
+            and record.msg == "Exception during reset or similar"
+            and record.exc_info
+            and self.pattern.search(str(record.exc_info[1]))):
+
+            # Log this as a warning instead of error, to cut down on our noise.
+            record.levelno = logging.WARNING
+            record.levelname = 'WARNING'
+
+        return True
