@@ -2,36 +2,36 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2015 Lance Edgar
+#  Copyright © 2010-2017 Lance Edgar
 #
 #  This file is part of Rattail.
 #
 #  Rattail is free software: you can redistribute it and/or modify it under the
-#  terms of the GNU Affero General Public License as published by the Free
-#  Software Foundation, either version 3 of the License, or (at your option)
-#  any later version.
+#  terms of the GNU General Public License as published by the Free Software
+#  Foundation, either version 3 of the License, or (at your option) any later
+#  version.
 #
 #  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
 #  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
-#  more details.
+#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+#  details.
 #
-#  You should have received a copy of the GNU Affero General Public License
-#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
+#  You should have received a copy of the GNU General Public License along with
+#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
 #
 ################################################################################
 """
 Vendor invoice parser for Albert's Organics
 """
 
-from __future__ import unicode_literals
+from __future__ import unicode_literals, absolute_import
 
 import csv
 import datetime
 
-from rattail.vendors.invoices import InvoiceParser
-from rattail.db.batch.vendorinvoice import VendorInvoiceRow
+from rattail.db import model
 from rattail.gpc import GPC
+from rattail.vendors.invoices import InvoiceParser
 
 
 class AlbertsInvoiceParser(InvoiceParser):
@@ -53,7 +53,7 @@ class AlbertsInvoiceParser(InvoiceParser):
             reader = csv.DictReader(f)
             for data in reader:
 
-                row = VendorInvoiceRow()
+                row = model.VendorInvoiceRow()
                 row.upc = GPC(data['UPCPLU'].replace('-', '').replace(' ', ''))
                 row.vendor_code = data['ItemNumber']
                 row.brand_name = data['BrandName']
