@@ -2,36 +2,33 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2014 Lance Edgar
+#  Copyright © 2010-2017 Lance Edgar
 #
 #  This file is part of Rattail.
 #
 #  Rattail is free software: you can redistribute it and/or modify it under the
-#  terms of the GNU Affero General Public License as published by the Free
-#  Software Foundation, either version 3 of the License, or (at your option)
-#  any later version.
+#  terms of the GNU General Public License as published by the Free Software
+#  Foundation, either version 3 of the License, or (at your option) any later
+#  version.
 #
 #  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
 #  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
-#  more details.
+#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+#  details.
 #
-#  You should have received a copy of the GNU Affero General Public License
-#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
+#  You should have received a copy of the GNU General Public License along with
+#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
 #
 ################################################################################
 """
 Core Stuff
 """
 
-from __future__ import unicode_literals
+from __future__ import unicode_literals, absolute_import
 
 from uuid import uuid1
 
 
-__all__ = ['Object', 'get_uuid']
-
-
 # Arbitrary object used to explicitly denote an unspecified value, e.g. for
 # comparison to an argument within a function signature.
 UNSPECIFIED = object()
@@ -60,18 +57,6 @@ class Object(object):
         if hasattr(self, key):
             return getattr(self, key)
 
-    def __str__(self):
-        """
-        Magic string method.
-
-        This leverages :meth:`__unicode__()` if it exists; otherwise falls back
-        to ``repr(self)``.
-        """
-
-        if hasattr(self, '__unicode__'):
-            return str(unicode(self))
-        return repr(self)
-
 
 def get_uuid():
     """
