@@ -2,22 +2,22 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2014 Lance Edgar
+#  Copyright © 2010-2017 Lance Edgar
 #
 #  This file is part of Rattail.
 #
 #  Rattail is free software: you can redistribute it and/or modify it under the
-#  terms of the GNU Affero General Public License as published by the Free
-#  Software Foundation, either version 3 of the License, or (at your option)
-#  any later version.
+#  terms of the GNU General Public License as published by the Free Software
+#  Foundation, either version 3 of the License, or (at your option) any later
+#  version.
 #
 #  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
 #  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
-#  more details.
+#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+#  details.
 #
-#  You should have received a copy of the GNU Affero General Public License
-#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
+#  You should have received a copy of the GNU General Public License along with
+#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
 #
 ################################################################################
 """
@@ -59,8 +59,9 @@ class UTF8Recoder(object):
        This class was stolen from the Python 2.7 documentation.
     """
 
-    def __init__(self, fileobj, encoding):
-        self.reader = codecs.getreader(encoding)(fileobj)
+    def __init__(self, fileobj, encoding, errors='strict'):
+        self.errors = errors
+        self.reader = codecs.getreader(encoding)(fileobj, errors=self.errors)
 
     def __iter__(self):
         return self
@@ -78,8 +79,8 @@ class UnicodeReader(object):
        This class was stolen from the Python 2.7 documentation.
     """
 
-    def __init__(self, fileobj, dialect=csv.excel, encoding='utf_8', **kwargs):
-        fileobj = UTF8Recoder(fileobj, encoding)
+    def __init__(self, fileobj, dialect=csv.excel, encoding='utf_8', errors='strict', **kwargs):
+        fileobj = UTF8Recoder(fileobj, encoding, errors=errors)
         self.reader = csv.reader(fileobj, dialect=dialect, **kwargs)
 
     def __iter__(self):
@@ -96,8 +97,8 @@ class UnicodeDictReader(object):
     encoded in the given encoding.
     """
 
-    def __init__(self, fileobj, dialect=csv.excel, encoding='utf_8', **kwargs):
-        fileobj = UTF8Recoder(fileobj, encoding)
+    def __init__(self, fileobj, dialect=csv.excel, encoding='utf_8', errors='strict', **kwargs):
+        fileobj = UTF8Recoder(fileobj, encoding, errors=errors)
         self.reader = csv.reader(fileobj, dialect=dialect, **kwargs)
         self.header = self.reader.next()
 
