@@ -21,6 +21,8 @@ class Daemon(object):
         self.stdin = stdin
         self.stdout = stdout
         self.stderr = stderr
+        if config:
+            self.enum = config.get_enum()
 
     def daemonize(self):
         """
