@@ -1,5 +1,5 @@
-# -*- coding: utf-8 -*-
-# -*- coding: utf-8 -*-
+# -*- coding: utf-8; -*-
+# -*- coding: utf-8; -*-
 """
 Handler for ${model_title} batches
 """
@@ -7,7 +7,7 @@ Handler for ${model_title} batches
 from __future__ import unicode_literals, absolute_import
 
 from rattail.db import model
-from rattail.db.batch.handler import FileBatchHandler
+from rattail.batch import FileBatchHandler
 
 
 class ${model_name}Handler(FileBatchHandler):
@@ -15,7 +15,6 @@ class ${model_name}Handler(FileBatchHandler):
     Handler for ${model_title} batches.
     """
     batch_model_class = model.${model_name}
-    show_progress = True
 
     def refresh_data(self, session, batch, progress=None):
         """
