@@ -1,11 +1,13 @@
-# -*- coding: utf-8 -*-
-# -*- coding: utf-8 -*-
+# -*- coding: utf-8; -*-
+# -*- coding: utf-8; -*-
 """
 Views for ${model_title} batches
 """
 
 from __future__ import unicode_literals, absolute_import
 
+import six
+
 from rattail.db import model
 
 from tailbone.views.batch import FileBatchMasterView
@@ -19,14 +21,14 @@ class ${model_name}View(FileBatchMasterView):
     """
     model_class = model.${model_name}
     batch_handler_class = ${model_name}Handler
-    batch_row_class = model.${model_name}Row
+    model_row_class = model.${model_name}Row
     model_title = "${model_title} Batch"
     model_title_plural = "${model_title} Batches"
     route_prefix = 'batch.${model_name.lower()}s'
     url_prefix = '/batch/${table_name.replace('_', '-')}'
 
     def get_instance_title(self, batch):
-        return unicode(batch.vendor)
+        return six.text_type(batch.vendor)
 
     def configure_grid(self, g):
         g.joiners['vendor'] = lambda q: q.join(model.Vendor)
