@@ -1,23 +1,23 @@
-# -*- coding: utf-8 -*-
+# -*- coding: utf-8; -*-
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2016 Lance Edgar
+#  Copyright © 2010-2017 Lance Edgar
 #
 #  This file is part of Rattail.
 #
 #  Rattail is free software: you can redistribute it and/or modify it under the
-#  terms of the GNU Affero General Public License as published by the Free
-#  Software Foundation, either version 3 of the License, or (at your option)
-#  any later version.
+#  terms of the GNU General Public License as published by the Free Software
+#  Foundation, either version 3 of the License, or (at your option) any later
+#  version.
 #
 #  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
 #  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
-#  more details.
+#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+#  details.
 #
-#  You should have received a copy of the GNU Affero General Public License
-#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
+#  You should have received a copy of the GNU General Public License along with
+#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
 #
 ################################################################################
 """
@@ -79,13 +79,16 @@ class DataSyncProfile(ConfigProfile):
     def normalize_consumers(self, raw_consumers):
         # TODO: This will do for now, but isn't as awesome as it probably could be.
         default_isolated = self._config_boolean('consumers.isolated', default=True)
+        default_runas = self._config_string('consumers.runas')
         consumers = []
         for key in raw_consumers:
-            consumer_spec = self._config_string('consumer.{0}'.format(key))
-            dbkey = self._config_string('consumer.{0}.db'.format(key), default=key)
-            consumer = load_object(consumer_spec)(self.config, key, dbkey=dbkey)
+            consumer_spec = self._config_string('consumer.{}'.format(key))
+            dbkey = self._config_string('consumer.{}.db'.format(key), default=key)
+            runas = self._config_string('consumer.{}.runas'.format(key))
+            consumer = load_object(consumer_spec)(self.config, key, dbkey=dbkey,
+                                                  runas=runas or default_runas)
             consumer.spec = consumer_spec
-            consumer.isolated = self._config_boolean('consumer.{0}.isolated'.format(key),
+            consumer.isolated = self._config_boolean('consumer.{}.isolated'.format(key),
                                                      default=default_isolated)
             consumers.append(consumer)
         return consumers
