@@ -1,23 +1,23 @@
-# -*- coding: utf-8 -*-
+# -*- coding: utf-8; -*-
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2016 Lance Edgar
+#  Copyright © 2010-2017 Lance Edgar
 #
 #  This file is part of Rattail.
 #
 #  Rattail is free software: you can redistribute it and/or modify it under the
-#  terms of the GNU Affero General Public License as published by the Free
-#  Software Foundation, either version 3 of the License, or (at your option)
-#  any later version.
+#  terms of the GNU General Public License as published by the Free Software
+#  Foundation, either version 3 of the License, or (at your option) any later
+#  version.
 #
 #  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
 #  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
-#  more details.
+#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+#  details.
 #
-#  You should have received a copy of the GNU Affero General Public License
-#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
+#  You should have received a copy of the GNU General Public License along with
+#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
 #
 ################################################################################
 """
@@ -27,6 +27,7 @@ Authentication & Authorization
 from __future__ import unicode_literals, absolute_import
 
 import bcrypt
+import six
 
 from rattail.db import model
 
@@ -42,9 +43,12 @@ def authenticate_user(session, username, password):
         .filter_by(username=username)\
         .first()
     if user and user.active and user.password is not None:
-        # Apparently bcrypt's hashpw() doesn't like Unicode?
+        # apparently bcrypt's hashpw() doesn't like Unicode
+        if isinstance(password, six.text_type):
+            password = password.encode('utf_8')
+        salt = user.salt.encode('utf_8')
         try:
-            authenticated = (bcrypt.hashpw(password, user.salt) == user.password)
+            authenticated = (bcrypt.hashpw(password, salt) == user.password)
         except UnicodeEncodeError:
             authenticated = False
         if authenticated:
@@ -56,7 +60,9 @@ def set_user_password(user, password):
     """
     Set a user's password.
     """
-
+    # apparently bcrypt's hashpw() doesn't like Unicode
+    if isinstance(password, six.text_type):
+        password = password.encode('utf_8')
     user.salt = bcrypt.gensalt()
     user.password = bcrypt.hashpw(password, user.salt)
 
