@@ -1,30 +1,30 @@
-# -*- coding: utf-8 -*-
+# -*- coding: utf-8; -*-
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2015 Lance Edgar
+#  Copyright © 2010-2017 Lance Edgar
 #
 #  This file is part of Rattail.
 #
 #  Rattail is free software: you can redistribute it and/or modify it under the
-#  terms of the GNU Affero General Public License as published by the Free
-#  Software Foundation, either version 3 of the License, or (at your option)
-#  any later version.
+#  terms of the GNU General Public License as published by the Free Software
+#  Foundation, either version 3 of the License, or (at your option) any later
+#  version.
 #
 #  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
 #  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
-#  more details.
+#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+#  details.
 #
-#  You should have received a copy of the GNU Affero General Public License
-#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
+#  You should have received a copy of the GNU General Public License along with
+#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
 #
 ################################################################################
 """
 Database Configuration
 """
 
-from __future__ import unicode_literals
+from __future__ import unicode_literals, absolute_import
 
 import logging
 
@@ -134,9 +134,8 @@ def configure_session(config, session):
     Configure a session factory or instance.  Currently all this does is
     install the hook to record changes, if config so dictates.
     """
-    from rattail.db.changes import record_changes
-
     if config.getbool('rattail.db', 'changes.record', usedb=False):
+        from rattail.db.changes import record_changes
         record_changes(session, config=config)
 
 
@@ -144,18 +143,20 @@ def configure_versioning(config, manager=None, plugins=None, **kwargs):
     """
     Configure Continuum versioning.
     """
-    if not config.getbool('rattail.db', 'versioning.enabled', default=False, usedb=False):
+    if not config.versioning_enabled():
         return
 
     try:
         import sqlalchemy_continuum as continuum
+        from sqlalchemy_continuum.plugins import TransactionMetaPlugin
         from rattail.db.continuum import versioning_manager, RattailPlugin
     except ImportError as error:
         raise SQLAlchemyNotInstalled(error)
     else:
-        if manager is None:
-            manager = versioning_manager
-        if plugins is None:
-            plugins = [RattailPlugin()]
+        kwargs['manager'] = manager or versioning_manager
+        if plugins:
+            kwargs['plugins'] = plugins
+        else:
+            kwargs['plugins'] = [TransactionMetaPlugin(), RattailPlugin()]
         log.info("enabling Continuum versioning")
-        continuum.make_versioned(manager=manager, plugins=plugins, **kwargs)
+        continuum.make_versioned(**kwargs)
