@@ -2,35 +2,37 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2016 Lance Edgar
+#  Copyright © 2010-2017 Lance Edgar
 #
 #  This file is part of Rattail.
 #
 #  Rattail is free software: you can redistribute it and/or modify it under the
-#  terms of the GNU Affero General Public License as published by the Free
-#  Software Foundation, either version 3 of the License, or (at your option)
-#  any later version.
+#  terms of the GNU General Public License as published by the Free Software
+#  Foundation, either version 3 of the License, or (at your option) any later
+#  version.
 #
 #  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
 #  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
-#  more details.
+#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+#  details.
 #
-#  You should have received a copy of the GNU Affero General Public License
-#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
+#  You should have received a copy of the GNU General Public License along with
+#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
 #
 ################################################################################
 """
-Data Models
+Core Data Models
 """
 
 from __future__ import unicode_literals, absolute_import
 
 import re
 
+import six
 import sqlalchemy as sa
 from sqlalchemy import orm
 from sqlalchemy.ext.declarative import declarative_base
+from sqlalchemy.ext.associationproxy import association_proxy
 
 from rattail.core import Object
 
@@ -61,22 +63,37 @@ class ModelBase(Object):
         # convert "CamelCase" to "Camel Case"
         return re.sub(r'([a-z])([A-Z])', r'\g<1> \g<2>', cls.__name__)
 
+    @classmethod
+    def get_model_title_plural(cls):
+        """
+        Return a "humanized" version of the plural model name, for display in templates etc.
+        """
+        if hasattr(cls, 'model_title_plural'):
+            return cls.model_title_plural
+        return '{}s'.format(cls.get_model_title())
+
+    @classmethod
+    def make_proxy(cls, main_class, extension, name):
+        setattr(main_class, name, association_proxy(extension, name,
+                                                    creator=lambda value: cls(**{name: value}),
+                                                    getset_factory=getset_factory))
+
 
 Base = declarative_base(cls=ModelBase)
 
 
+@six.python_2_unicode_compatible
 class Setting(Base):
     """
-    Represents a setting stored within the database.
+    Represents a "raw" config setting stored within the database.
     """
-
     __tablename__ = 'setting'
 
     name = sa.Column(sa.String(length=255), primary_key=True)
     value = sa.Column(sa.Text())
 
-    def __unicode__(self):
-        return unicode(self.name or '')
+    def __str__(self):
+        return self.name or ''
 
 
 class Change(Base):
