@@ -1,23 +1,23 @@
-# -*- coding: utf-8 -*-
+# -*- coding: utf-8; -*-
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2015 Lance Edgar
+#  Copyright © 2010-2017 Lance Edgar
 #
 #  This file is part of Rattail.
 #
 #  Rattail is free software: you can redistribute it and/or modify it under the
-#  terms of the GNU Affero General Public License as published by the Free
-#  Software Foundation, either version 3 of the License, or (at your option)
-#  any later version.
+#  terms of the GNU General Public License as published by the Free Software
+#  Foundation, either version 3 of the License, or (at your option) any later
+#  version.
 #
 #  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
 #  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
-#  more details.
+#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+#  details.
 #
-#  You should have received a copy of the GNU Affero General Public License
-#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
+#  You should have received a copy of the GNU General Public License along with
+#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
 #
 ################################################################################
 """
@@ -26,16 +26,17 @@ Data Models for Customers
 
 from __future__ import unicode_literals, absolute_import
 
+import six
 import sqlalchemy as sa
 from sqlalchemy import orm
 from sqlalchemy.ext.associationproxy import association_proxy
 from sqlalchemy.ext.orderinglist import ordering_list
 
-from .core import Base, uuid_column, getset_factory
-from .contact import PhoneNumber, EmailAddress
-from .people import Person
+from rattail.db.model import Base, uuid_column, getset_factory
+from rattail.db.model import PhoneNumber, EmailAddress, MailingAddress, Person
 
 
+@six.python_2_unicode_compatible
 class Customer(Base):
     """
     Represents a customer account.
@@ -46,12 +47,34 @@ class Customer(Base):
     __versioned__ = {}
 
     uuid = uuid_column()
-    id = sa.Column(sa.String(length=20))
+
+    id = sa.Column(sa.String(length=20), nullable=True, doc="""
+    String ID for the customer, if known/relevant.  This may or may not
+    correspond to the :attr:`number`, depending on your system.
+    """)
+
+    number = sa.Column(sa.Integer(), nullable=True, doc="""
+    Customer number, if known/relevant.  This may or may not correspond to the
+    :attr:`id`, depending on your system.
+    """)
+
     name = sa.Column(sa.String(length=255))
     email_preference = sa.Column(sa.Integer())
 
-    def __unicode__(self):
-        return unicode(self.name or self.person or '')
+    active_in_pos = sa.Column(sa.Boolean(), nullable=True, doc="""
+    Whether or not the customer account should be "active" within the POS
+    system, if applicable.  Whether/how this field is populated and/or
+    leveraged are up to your system.
+    """)
+
+    active_in_pos_sticky = sa.Column(sa.Boolean(), nullable=False, default=False, doc="""
+    Whether or not the customer account should *always* be "active" within the
+    POS system.  This field may be useful if :attr:`active_in_pos` gets set
+    dynamically.
+    """)
+
+    def __str__(self):
+        return self.name or ''
 
     def add_email_address(self, address, type='Home'):
         email = CustomerEmailAddress(address=address, type=type)
@@ -116,6 +139,32 @@ Customer.email = orm.relationship(
     viewonly=True)
 
 
+class CustomerMailingAddress(MailingAddress):
+    """
+    Represents a mailing address for a customer
+    """
+    __mapper_args__ = {'polymorphic_identity': 'Customer'}
+
+Customer.addresses = orm.relationship(
+    CustomerMailingAddress,
+    backref='customer',
+    primaryjoin=CustomerMailingAddress.parent_uuid == Customer.uuid,
+    foreign_keys=[CustomerMailingAddress.parent_uuid],
+    collection_class=ordering_list('preference', count_from=1),
+    order_by=CustomerMailingAddress.preference,
+    cascade='all, delete-orphan')
+
+Customer.address = orm.relationship(
+    CustomerMailingAddress,
+    primaryjoin=sa.and_(
+        CustomerMailingAddress.parent_uuid == Customer.uuid,
+        CustomerMailingAddress.preference == 1),
+    foreign_keys=[CustomerMailingAddress.parent_uuid],
+    uselist=False,
+    viewonly=True)
+
+
+@six.python_2_unicode_compatible
 class CustomerGroup(Base):
     """
     Represents an arbitrary group to which customers may belong.
@@ -127,8 +176,8 @@ class CustomerGroup(Base):
     id = sa.Column(sa.String(length=20))
     name = sa.Column(sa.String(length=255))
 
-    def __unicode__(self):
-        return unicode(self.name or '')
+    def __str__(self):
+        return self.name or ''
 
 
 class CustomerGroupAssignment(Base):
@@ -139,7 +188,8 @@ class CustomerGroupAssignment(Base):
     __table_args__ = (
         sa.ForeignKeyConstraint(['group_uuid'], ['customer_group.uuid'], name='customer_x_group_fk_group'),
         sa.ForeignKeyConstraint(['customer_uuid'], ['customer.uuid'], name='customer_x_group_fk_customer'),
-        )
+    )
+    __versioned__ = {}
 
     uuid = uuid_column()
     customer_uuid = sa.Column(sa.String(length=32), nullable=False)
@@ -169,7 +219,8 @@ class CustomerPerson(Base):
     __table_args__ = (
         sa.ForeignKeyConstraint(['customer_uuid'], ['customer.uuid'], name='customer_x_person_fk_customer'),
         sa.ForeignKeyConstraint(['person_uuid'], ['person.uuid'], name='customer_x_person_fk_person'),
-        )
+    )
+    __versioned__ = {}
 
     uuid = uuid_column()
     customer_uuid = sa.Column(sa.String(length=32), nullable=False)
