@@ -2,36 +2,38 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2015 Lance Edgar
+#  Copyright © 2010-2017 Lance Edgar
 #
 #  This file is part of Rattail.
 #
 #  Rattail is free software: you can redistribute it and/or modify it under the
-#  terms of the GNU Affero General Public License as published by the Free
-#  Software Foundation, either version 3 of the License, or (at your option)
-#  any later version.
+#  terms of the GNU General Public License as published by the Free Software
+#  Foundation, either version 3 of the License, or (at your option) any later
+#  version.
 #
 #  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
 #  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
-#  more details.
+#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+#  details.
 #
-#  You should have received a copy of the GNU Affero General Public License
-#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
+#  You should have received a copy of the GNU General Public License along with
+#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
 #
 ################################################################################
 """
 Data Models for Organization etc.
 """
 
-from __future__ import unicode_literals
+from __future__ import unicode_literals, absolute_import
 
+import six
 import sqlalchemy as sa
-from sqlalchemy.orm import relationship
+from sqlalchemy import orm
 
-from .core import Base, uuid_column
+from rattail.db.model import Base, uuid_column
 
 
+@six.python_2_unicode_compatible
 class Department(Base):
     """
     Represents an organizational department.
@@ -43,10 +45,11 @@ class Department(Base):
     number = sa.Column(sa.Integer())
     name = sa.Column(sa.String(length=30))
 
-    def __unicode__(self):
-        return unicode(self.name or '')
+    def __str__(self):
+        return self.name or ''
 
 
+@six.python_2_unicode_compatible
 class Subdepartment(Base):
     """
     Represents an organizational subdepartment.
@@ -54,7 +57,7 @@ class Subdepartment(Base):
     __tablename__ = 'subdepartment'
     __table_args__ = (
         sa.ForeignKeyConstraint(['department_uuid'], ['department.uuid'], name='subdepartment_fk_department'),
-        )
+    )
     __versioned__ = {}
 
     uuid = uuid_column()
@@ -62,16 +65,17 @@ class Subdepartment(Base):
     name = sa.Column(sa.String(length=30))
     department_uuid = sa.Column(sa.String(length=32))
 
-    def __unicode__(self):
+    def __str__(self):
         return self.name or ''
 
 
-Department.subdepartments = relationship(
+Department.subdepartments = orm.relationship(
     Subdepartment,
     backref='department',
     order_by=Subdepartment.name)
 
 
+@six.python_2_unicode_compatible
 class Category(Base):
     """
     Represents an organizational category for products.
@@ -89,12 +93,13 @@ class Category(Base):
     name = sa.Column(sa.String(length=50))
     department_uuid = sa.Column(sa.String(length=32))
 
-    department = relationship(Department)
+    department = orm.relationship(Department)
 
-    def __unicode__(self):
+    def __str__(self):
         return self.name or ''
 
 
+@six.python_2_unicode_compatible
 class Family(Base):
     """
     Represents an organizational family for products.
@@ -106,15 +111,11 @@ class Family(Base):
     code = sa.Column(sa.Integer())
     name = sa.Column(sa.String(length=50))
 
-    products = relationship(
-        u'Product', back_populates=u'family', doc=u"""\
-Collection of :class:`Product` instances which associate with this family.
-""")
-
-    def __unicode__(self):
-        return unicode(self.name or '')
+    def __str__(self):
+        return self.name or ''
 
 
+@six.python_2_unicode_compatible
 class ReportCode(Base):
     """
     Represents an organizational "report code" for products.
@@ -122,24 +123,20 @@ class ReportCode(Base):
     __tablename__ = 'report_code'
     __table_args__ = (
         sa.UniqueConstraint('code', name='report_code_uq_code'),
-        )
+    )
     __versioned__ = {}
 
     uuid = uuid_column()
     code = sa.Column(sa.Integer(), nullable=False)
     name = sa.Column(sa.String(length=100), nullable=True)
 
-    products = relationship(
-        u'Product', back_populates=u'report_code', doc=u"""\
-Collection of :class:`Product` instances which associate with this report code.
-""")
-
-    def __unicode__(self):
+    def __str__(self):
         if not self.code:
             return ''
         return "{} - {}".format(self.code, self.name or '')
 
 
+@six.python_2_unicode_compatible
 class DepositLink(Base):
     """
     Represents a bottle or similar deposit, to which an item may be linked.
@@ -147,15 +144,19 @@ class DepositLink(Base):
     __tablename__ = 'deposit_link'
     __table_args__ = (
         sa.UniqueConstraint('code', name='deposit_link_uq_code'),
-        )
+    )
     __versioned__ = {}
 
     uuid = uuid_column()
-    code = sa.Column(sa.Integer(), nullable=False)
+
+    code = sa.Column(sa.String(length=20), nullable=False, doc="""
+    Unique "code" for the deposit link.
+    """)
+
     description = sa.Column(sa.String(length=255), nullable=True)
     amount = sa.Column(sa.Numeric(precision=5, scale=2), nullable=False)
 
-    def __unicode__(self):
-        if not (self.description and self.amount):
-            return ""
-        return "{0} (${1:0.2f})".format(self.description, self.amount)
+    def __str__(self):
+        if self.description:
+            return self.description
+        return "${:0.2f}".format(self.amount)
