@@ -2,22 +2,22 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2016 Lance Edgar
+#  Copyright © 2010-2017 Lance Edgar
 #
 #  This file is part of Rattail.
 #
 #  Rattail is free software: you can redistribute it and/or modify it under the
-#  terms of the GNU Affero General Public License as published by the Free
-#  Software Foundation, either version 3 of the License, or (at your option)
-#  any later version.
+#  terms of the GNU General Public License as published by the Free Software
+#  Foundation, either version 3 of the License, or (at your option) any later
+#  version.
 #
 #  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
 #  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
-#  more details.
+#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+#  details.
 #
-#  You should have received a copy of the GNU Affero General Public License
-#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
+#  You should have received a copy of the GNU General Public License along with
+#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
 #
 ################################################################################
 """
@@ -28,6 +28,7 @@ from __future__ import unicode_literals, absolute_import
 
 import datetime
 
+import six
 import sqlalchemy as sa
 from sqlalchemy import orm
 from sqlalchemy.ext.orderinglist import ordering_list
@@ -48,6 +49,7 @@ def get_person_display_name_from_context(context):
     return normalize_full_name(first_name, last_name)
 
 
+@six.python_2_unicode_compatible
 class Person(Base):
     """
     Represents a real, living and breathing person.
@@ -56,17 +58,20 @@ class Person(Base):
     deceased.)
     """
     __tablename__ = 'person'
-    __versioned__ = {}
+    __versioned__ = {
+        'exclude': ['modified'],
+    }
 
     uuid = uuid_column()
     first_name = sa.Column(sa.String(length=50), nullable=True)
     middle_name = sa.Column(sa.String(length=50), nullable=True)
     last_name = sa.Column(sa.String(length=50), nullable=True)
-    display_name = sa.Column(sa.String(length=100), default=get_person_display_name_from_context)
+    # TODO: rename this to 'full_name' at some point...?
+    display_name = sa.Column(sa.String(length=100), nullable=True)
     modified = sa.Column(sa.DateTime(), nullable=True, onupdate=datetime.datetime.utcnow)
 
-    def __unicode__(self):
-        return unicode(self.display_name or '')
+    def __str__(self):
+        return self.display_name or ''
 
     def add_email_address(self, address, type='Home'):
         email = PersonEmailAddress(address=address, type=type)
