@@ -1,39 +1,42 @@
-# -*- coding: utf-8 -*-
+# -*- coding: utf-8; -*-
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2015 Lance Edgar
+#  Copyright © 2010-2017 Lance Edgar
 #
 #  This file is part of Rattail.
 #
 #  Rattail is free software: you can redistribute it and/or modify it under the
-#  terms of the GNU Affero General Public License as published by the Free
-#  Software Foundation, either version 3 of the License, or (at your option)
-#  any later version.
+#  terms of the GNU General Public License as published by the Free Software
+#  Foundation, either version 3 of the License, or (at your option) any later
+#  version.
 #
 #  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
 #  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
-#  more details.
+#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+#  details.
 #
-#  You should have received a copy of the GNU Affero General Public License
-#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
+#  You should have received a copy of the GNU General Public License along with
+#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
 #
 ################################################################################
 """
 Data Models for Users & Permissions
 """
 
-from __future__ import unicode_literals
+from __future__ import unicode_literals, absolute_import
 
+import datetime
+
+import six
 import sqlalchemy as sa
-from sqlalchemy.orm import relationship
+from sqlalchemy import orm
 from sqlalchemy.ext.associationproxy import association_proxy
 
-from .core import Base, uuid_column, getset_factory
-from .people import Person
+from rattail.db.model import Base, uuid_column, getset_factory, Person
  
 
+@six.python_2_unicode_compatible
 class Role(Base):
     """
     Represents a role within the system; used to manage permissions.
@@ -47,10 +50,17 @@ class Role(Base):
     uuid = uuid_column()
     name = sa.Column(sa.String(length=25), nullable=False)
 
-    def __unicode__(self):
-        return unicode(self.name or '')
+    session_timeout = sa.Column(sa.Integer(), nullable=True, doc="""
+    Optional session timeout value for the role, in seconds.  If this is set to
+    zero, the role's users will have no session timeout.  A value of ``None``
+    means the role has no say in the timeout.
+    """)
+
+    def __str__(self):
+        return self.name or ''
 
 
+@six.python_2_unicode_compatible
 class Permission(Base):
     """
     Represents permission a role has to do a particular thing.
@@ -59,15 +69,16 @@ class Permission(Base):
     __table_args__ = (
         sa.ForeignKeyConstraint(['role_uuid'], ['role.uuid'], name='permission_fk_role'),
         )
+    # __versioned__ = {}
 
     role_uuid = sa.Column(sa.String(length=32), primary_key=True)
     permission = sa.Column(sa.String(length=50), primary_key=True)
 
-    def __unicode__(self):
-        return unicode(self.permission or '')
+    def __str__(self):
+        return self.permission or ''
 
 
-Role._permissions = relationship(
+Role._permissions = orm.relationship(
     Permission, backref='role',
     cascade='save-update, merge, delete, delete-orphan')
 
@@ -77,6 +88,7 @@ Role.permissions = association_proxy(
     getset_factory=getset_factory)
 
 
+@six.python_2_unicode_compatible
 class User(Base):
     """
     Represents a user of the system.
@@ -97,15 +109,23 @@ class User(Base):
     salt = sa.Column(sa.String(length=29))
     person_uuid = sa.Column(sa.String(length=32))
 
-    active = sa.Column(sa.Boolean(), nullable=False, default=True)
-    """
+    active = sa.Column(sa.Boolean(), nullable=False, default=True, doc="""
     Whether the user is active, e.g. allowed to log in via the UI.
-    """
+    """)
 
-    def __unicode__(self):
-        if self.person and unicode(self.person):
-            return unicode(self.person)
-        return unicode(self.username)
+    active_sticky = sa.Column(sa.Boolean(), nullable=True, doc="""
+    Optional flag, motivation behind which is as follows: If you import user
+    accounts from another system, esp. on a regular basis, you might be keeping
+    the :attr:`active` flag in sync along with that.  But in some cases you
+    might want to *not* keep the active flag in sync, for certain accounts.
+    Hence this "active sticky" flag, which may be used to mark certain accounts
+    as off-limits from the general active flag sync.
+    """)
+
+    def __str__(self):
+        if self.person and str(self.person):
+            return str(self.person)
+        return self.username or ''
 
     @property
     def display_name(self):
@@ -127,6 +147,25 @@ class User(Base):
         if self.person:
             return self.person.employee
 
+    def get_short_name(self):
+        """
+        Returns "short name" for the user.  This is for convenience of mobile
+        view, at least...
+        """
+        # TODO: this should reference employee.short_name
+        employee = self.employee
+        if employee and employee.display_name:
+            return employee.display_name
+
+        person = self.person
+        if person:
+            if person.first_name and person.last_name:
+                return "{} {}.".format(person.first_name, person.last_name[0])
+            if person.first_name:
+                return person.first_name
+
+        return self.username
+
     def get_email_address(self):
         """
         Returns the primary email address for the user (as unicode string), or
@@ -152,13 +191,27 @@ class User(Base):
         """
         return self.get_email_address()
 
+    def is_admin(self):
+        """
+        Convenience method to determine if current user is a member of the
+        Administrator role.
+        """
+        from rattail.db.auth import administrator_role
 
-User.person = relationship(
+        session = orm.object_session(self)
+        return administrator_role(session) in self.roles
+
+    def record_event(self, type_code, **kwargs):
+        kwargs['type_code'] = type_code
+        self.events.append(UserEvent(**kwargs))
+
+
+User.person = orm.relationship(
     Person,
     back_populates='user',
     uselist=False)
 
-Person.user = relationship(
+Person.user = orm.relationship(
     User,
     back_populates='person',
     uselist=False)
@@ -173,13 +226,14 @@ class UserRole(Base):
         sa.ForeignKeyConstraint(['user_uuid'], ['user.uuid'], name='user_x_role_fk_user'),
         sa.ForeignKeyConstraint(['role_uuid'], ['role.uuid'], name='user_x_role_fk_role'),
         )
+    __versioned__ = {}
 
     uuid = uuid_column()
     user_uuid = sa.Column(sa.String(length=32), nullable=False)
     role_uuid = sa.Column(sa.String(length=32), nullable=False)
 
 
-Role._users = relationship(
+Role._users = orm.relationship(
     UserRole, backref='role',
     cascade='all, delete-orphan')
 
@@ -188,7 +242,7 @@ Role.users = association_proxy(
     creator=lambda u: UserRole(user=u),
     getset_factory=getset_factory)
 
-User._roles = relationship(
+User._roles = orm.relationship(
     UserRole, backref='user',
     cascade='all, delete-orphan')
 
@@ -196,3 +250,35 @@ User.roles = association_proxy(
     '_roles', 'role',
     creator=lambda r: UserRole(role=r),
     getset_factory=getset_factory)
+
+
+class UserEvent(Base):
+    """
+    Represents an event associated with a user.
+    """
+    __tablename__ = 'user_event'
+    __table_args__ = (
+        sa.ForeignKeyConstraint(['user_uuid'], ['user.uuid'], name='user_x_role_fk_user'),
+    )
+
+    uuid = uuid_column()
+
+    user_uuid = sa.Column(sa.String(length=32), nullable=False)
+    user = orm.relationship(
+        User,
+        doc="""
+        Reference to the user whose event this is.
+        """,
+        backref=orm.backref(
+            'events',
+            doc="""
+            Sequence of events for the user.
+            """))
+
+    type_code = sa.Column(sa.Integer(), nullable=False, doc="""
+    Type code for the event.
+    """)
+
+    occurred = sa.Column(sa.DateTime(), nullable=True, default=datetime.datetime.utcnow, doc="""
+    Timestamp at which the event occurred.
+    """)
