@@ -2,22 +2,22 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2016 Lance Edgar
+#  Copyright © 2010-2017 Lance Edgar
 #
 #  This file is part of Rattail.
 #
 #  Rattail is free software: you can redistribute it and/or modify it under the
-#  terms of the GNU Affero General Public License as published by the Free
-#  Software Foundation, either version 3 of the License, or (at your option)
-#  any later version.
+#  terms of the GNU General Public License as published by the Free Software
+#  Foundation, either version 3 of the License, or (at your option) any later
+#  version.
 #
 #  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
 #  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
-#  more details.
+#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+#  details.
 #
-#  You should have received a copy of the GNU Affero General Public License
-#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
+#  You should have received a copy of the GNU General Public License along with
+#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
 #
 ################################################################################
 """
@@ -54,12 +54,39 @@ class QuerySequence(object):
         self.query = query
 
     def __len__(self):
-        return self.query.count()
+        try:
+            return len(self.query)
+        except TypeError:
+            return self.query.count()
 
     def __iter__(self):
         return iter(self.query)
 
 
+class short_session(object):
+    """
+    Context manager for a short-lived database session
+    """
+
+    def __init__(self, session=None, Session=None):
+        self.session = session
+        self.Session = Session
+        self.private_session = not bool(session)
+
+    def __enter__(self):
+        if not self.session:
+            if not self.Session:
+                from rattail.db import Session
+                self.Session = Session
+            self.session = self.Session()
+        return self.session
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        if self.private_session:
+            self.session.close()
+            self.session = None
+
+
 def maxlen(attr):
     """
     Return the maximum length for the given attribute.
@@ -93,6 +120,18 @@ def make_topo_sortkey(model, metadata=None):
     return sortkey
 
 
+def make_full_description(brand_name, description, size):
+    """
+    Combine the given field values into a complete description.
+    """
+    fields = [
+        brand_name or '',
+        description or '',
+        size or '']
+    fields = [f.strip() for f in fields if f.strip()]
+    return ' '.join(fields)
+
+
 def normalize_full_name(first_name, last_name):
     """
     Normalize the given first and last name to a "full" name value.  The
