@@ -2,22 +2,22 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2016 Lance Edgar
+#  Copyright © 2010-2017 Lance Edgar
 #
 #  This file is part of Rattail.
 #
 #  Rattail is free software: you can redistribute it and/or modify it under the
-#  terms of the GNU Affero General Public License as published by the Free
-#  Software Foundation, either version 3 of the License, or (at your option)
-#  any later version.
+#  terms of the GNU General Public License as published by the Free Software
+#  Foundation, either version 3 of the License, or (at your option) any later
+#  version.
 #
 #  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
 #  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
-#  more details.
+#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+#  details.
 #
-#  You should have received a copy of the GNU Affero General Public License
-#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
+#  You should have received a copy of the GNU General Public License along with
+#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
 #
 ################################################################################
 """
@@ -52,6 +52,8 @@ class datasync_error_watcher_get_changes(Email):
             'watcher': watcher,
             'error': exc,
             'traceback': ''.join(format_exception(exc_type, exc, traceback)).strip(),
+            'datasync_url': '/datasyncchanges',
+            'attempts': 2,
         }
 
 
@@ -79,3 +81,18 @@ class filemon_action_error(Email):
             'error': exc,
             'traceback': ''.join(format_exception(exc_type, exc, traceback)).strip(),
         }
+
+
+class user_feedback(Email):
+    """
+    Sent when a user submits a Feedback form from the web UI.
+    """
+    default_subject = "User Feedback"
+
+    def sample_data(self, request):
+        return {
+            'user': None,
+            'user_name': "Fred Flintstone",
+            'referrer': request.route_url('home'),
+            'message': "Hey there,\n\njust wondered what the heck was going on with this site?  It's crap.\n\nFred",
+        }
