@@ -2,22 +2,22 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2014 Lance Edgar
+#  Copyright © 2010-2017 Lance Edgar
 #
 #  This file is part of Rattail.
 #
 #  Rattail is free software: you can redistribute it and/or modify it under the
-#  terms of the GNU Affero General Public License as published by the Free
-#  Software Foundation, either version 3 of the License, or (at your option)
-#  any later version.
+#  terms of the GNU General Public License as published by the Free Software
+#  Foundation, either version 3 of the License, or (at your option) any later
+#  version.
 #
 #  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
 #  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
-#  more details.
+#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+#  details.
 #
-#  You should have received a copy of the GNU Affero General Public License
-#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
+#  You should have received a copy of the GNU General Public License along with
+#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
 #
 ################################################################################
 """
@@ -144,7 +144,18 @@ PRICE_TYPE = {
     }
 
 
-PURCHASE_STATUS_NEW             = 1
+PURCHASE_BATCH_MODE_ORDERING            = 10
+PURCHASE_BATCH_MODE_RECEIVING           = 20
+PURCHASE_BATCH_MODE_COSTING             = 30
+
+PURCHASE_BATCH_MODE = {
+    PURCHASE_BATCH_MODE_ORDERING        : "ordering",
+    PURCHASE_BATCH_MODE_RECEIVING       : "receiving",
+    PURCHASE_BATCH_MODE_COSTING         : "invoicing",
+}
+
+
+PURCHASE_STATUS_NEW             = 1 # TODO: is this needed?
 PURCHASE_STATUS_ORDERED         = 10
 PURCHASE_STATUS_RECEIVED        = 20
 PURCHASE_STATUS_COSTED          = 30
@@ -154,11 +165,35 @@ PURCHASE_STATUS = {
     PURCHASE_STATUS_NEW         : "new/pending",
     PURCHASE_STATUS_ORDERED     : "ordered",
     PURCHASE_STATUS_RECEIVED    : "received",
-    PURCHASE_STATUS_COSTED      : "costed",
+    PURCHASE_STATUS_COSTED      : "invoiced",
     PURCHASE_STATUS_PAID        : "paid",
 }
 
 
+TEMPMON_APPLIANCE_TYPE_COOLER           = 1
+TEMPMON_APPLIANCE_TYPE_FREEZER          = 2
+
+TEMPMON_APPLIANCE_TYPE = {
+    TEMPMON_APPLIANCE_TYPE_COOLER       : "cooler",
+    TEMPMON_APPLIANCE_TYPE_FREEZER      : "freezer",
+}
+
+
+TEMPMON_PROBE_STATUS_GOOD_TEMP          = 1
+TEMPMON_PROBE_STATUS_LOW_TEMP           = 2
+TEMPMON_PROBE_STATUS_HIGH_TEMP          = 3
+TEMPMON_PROBE_STATUS_CRITICAL_TEMP      = 4
+TEMPMON_PROBE_STATUS_ERROR              = 5
+
+TEMPMON_PROBE_STATUS = {
+    TEMPMON_PROBE_STATUS_GOOD_TEMP      : "good temp",
+    TEMPMON_PROBE_STATUS_LOW_TEMP       : "low temp",
+    TEMPMON_PROBE_STATUS_HIGH_TEMP      : "high temp",
+    TEMPMON_PROBE_STATUS_CRITICAL_TEMP  : "critical temp",
+    TEMPMON_PROBE_STATUS_ERROR          : "error",
+}
+
+
 # These values are taken from the SIL standard.
 UNIT_OF_MEASURE_NONE                    = '00'
 UNIT_OF_MEASURE_EACH                    = '01'
@@ -329,6 +364,15 @@ UNIT_OF_MEASURE = {
     }
 
 
+USER_EVENT_LOGIN                = 1
+USER_EVENT_LOGOUT               = 2
+
+USER_EVENT = {
+    USER_EVENT_LOGIN            : "login",
+    USER_EVENT_LOGOUT           : "logout",
+}
+
+
 EMPLOYEE_STATUS_CURRENT         = 1
 EMPLOYEE_STATUS_FORMER          = 2
 
@@ -349,3 +393,6 @@ EMPLOYEE_STATUS = {
 #     VENDOR_CATALOG_COGNIZED     : "cognized",
 #     VENDOR_CATALOG_PROCESSED    : "processed",
 #     }
+
+
+TRAINWRECK_SYSTEM = {}
