@@ -2,22 +2,22 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2016 Lance Edgar
+#  Copyright © 2010-2017 Lance Edgar
 #
 #  This file is part of Rattail.
 #
 #  Rattail is free software: you can redistribute it and/or modify it under the
-#  terms of the GNU Affero General Public License as published by the Free
-#  Software Foundation, either version 3 of the License, or (at your option)
-#  any later version.
+#  terms of the GNU General Public License as published by the Free Software
+#  Foundation, either version 3 of the License, or (at your option) any later
+#  version.
 #
 #  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
 #  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
-#  more details.
+#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+#  details.
 #
-#  You should have received a copy of the GNU Affero General Public License
-#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
+#  You should have received a copy of the GNU General Public License along with
+#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
 #
 ################################################################################
 """
@@ -31,7 +31,7 @@ import re
 import tempfile
 
 from fabric.api import sudo, run, cd, local, env, settings, put as fab_put
-from fabric.contrib.files import exists, append, upload_template as fab_upload_template
+from fabric.contrib.files import exists, append, upload_template as fab_upload_template, is_link
 
 from mako.template import Template
 
@@ -71,10 +71,9 @@ def mkdir(paths, owner='root:root', mode=None):
         sudo('chmod {0} {1}'.format(mode, ' '.join(paths)))
 
 
-def make_system_user(name='rattail', home='/srv/rattail', uid=None, shell=None, alias=False):
+def make_system_user(name='rattail', home='/srv/rattail', uid=None, shell=None):
     """
     Make a new system user account, with the given home folder and shell path.
-    Optionally add a mail alias to root as well.
     """
     with settings(warn_only=True):
         result = sudo('getent passwd {0}'.format(name))
@@ -82,13 +81,6 @@ def make_system_user(name='rattail', home='/srv/rattail', uid=None, shell=None,
         uid = '--uid {0}'.format(uid) if uid else ''
         shell = '--shell {0}'.format(shell) if shell else ''
         sudo('adduser --system --home {0} --group {1} {2} {3}'.format(home, name, uid, shell))
-    if alias:
-        add_mail_alias(name)
-
-
-def add_mail_alias(local_name, aliased_to='root'):
-    append('/etc/aliases', '{}: {}'.format(local_name, aliased_to), use_sudo=True)
-    sudo('newaliases')
 
 
 def set_timezone(timezone):
@@ -96,15 +88,18 @@ def set_timezone(timezone):
     Set the system timezone to the given value, e.g. 'America/Chicago'.
     """
     sudo('echo {} >/etc/timezone'.format(timezone))
-    sudo('cp /usr/share/zoneinfo/{} /etc/localtime'.format(timezone))
+    if is_link('/etc/localtime'):
+        sudo('ln --symbolic --force /usr/share/zoneinfo/{} /etc/localtime'.format(timezone))
+    else:
+        sudo('cp /usr/share/zoneinfo/{} /etc/localtime'.format(timezone))
 
 
-def agent_sudo(cmd):
+def agent_sudo(cmd, user=None):
     """
     Run a 'sudo' command on the target server, with full agent forwarding. 
     """
     with settings(forward_agent=True):
-        sudo('SSH_AUTH_SOCK=$SSH_AUTH_SOCK {}'.format(cmd), shell=False)
+        sudo('SSH_AUTH_SOCK=$SSH_AUTH_SOCK {}'.format(cmd), shell=False, user=user)
 
 
 def put(local_path, remote_path, owner='root:root', **kwargs):
@@ -129,7 +124,7 @@ def upload_template(local_path, remote_path, owner='root:root', **kwargs):
     sudo('chown {0} {1}'.format(owner, remote_path))
 
 
-def upload_mako_template(local_path, remote_path, owner='root:root', context={}, **kwargs):
+def upload_mako_template(local_path, remote_path, context={}, **kwargs):
     """
     Render a local file as a Mako template, and upload the result to the server.
     """
@@ -200,6 +195,15 @@ class Deployer(object):
                 self.deploy('{}/meta.json'.format(localdir), 'meta.json')
                 self.deploy('{}/regr.json'.format(localdir), 'regr.json')
 
+    def soffice_daemon(self, local_path, name=None, register=True, start=True, **kwargs):
+        if name is None:
+            name = local_path.split('/')[-1]
+        self.deploy(local_path, '/etc/init.d/{}'.format(name), **kwargs)
+        if register:
+            sudo('update-rc.d {} defaults'.format(name))
+            if start:
+                sudo('service {} restart'.format(name))
+
 
 def make_deploy(deploy_path, last_segment='deploy'):
     """
