@@ -1,23 +1,23 @@
-# -*- coding: utf-8 -*-
+# -*- coding: utf-8; -*-
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2016 Lance Edgar
+#  Copyright © 2010-2017 Lance Edgar
 #
 #  This file is part of Rattail.
 #
 #  Rattail is free software: you can redistribute it and/or modify it under the
-#  terms of the GNU Affero General Public License as published by the Free
-#  Software Foundation, either version 3 of the License, or (at your option)
-#  any later version.
+#  terms of the GNU General Public License as published by the Free Software
+#  Foundation, either version 3 of the License, or (at your option) any later
+#  version.
 #
 #  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
 #  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
-#  more details.
+#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+#  details.
 #
-#  You should have received a copy of the GNU Affero General Public License
-#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
+#  You should have received a copy of the GNU General Public License along with
+#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
 #
 ################################################################################
 """
@@ -26,7 +26,7 @@ Fabric Library for MySQL
 
 from __future__ import unicode_literals, absolute_import
 
-from fabric.api import sudo, hide, get, put
+from fabric.api import sudo, hide, get, put, abort, local, run
 from fabric.contrib.files import sed
 
 from rattail.fablib import apt, make_deploy
@@ -163,3 +163,33 @@ def restore_db(name, source=None, user=None):
     create_db(name, user=user)
     sudo('mysql --execute="source {0}.sql" {0}'.format(name))
     sudo('rm {}.sql'.format(name))
+
+
+def clone_db(name, download, force=False):
+    """
+    Clone a MySQL database from a (presumably live) server
+
+    :param name: Name of the database.
+
+    :param force: Whether the target database should be forcibly dropped, if it
+       exists already.
+    """
+    if db_exists(name):
+       if force:
+           drop_db(name, checkfirst=False)
+       else:
+           abort("Database '{}' already exists! (pass force=true to override)".format(name))
+
+    create_db(name, checkfirst=False)
+
+    # obtain database dump from live server
+    download()
+
+    # upload database dump to target server
+    put('{}.sql.gz'.format(name))
+    local('rm {}.sql.gz'.format(name))
+
+    # restore database on target server
+    run('gunzip --force {}.sql.gz'.format(name))
+    sudo('mysql {0} < {0}.sql'.format(name))
+    run('rm {}.sql'.format(name))
