@@ -2,22 +2,22 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2014 Lance Edgar
+#  Copyright © 2010-2017 Lance Edgar
 #
 #  This file is part of Rattail.
 #
 #  Rattail is free software: you can redistribute it and/or modify it under the
-#  terms of the GNU Affero General Public License as published by the Free
-#  Software Foundation, either version 3 of the License, or (at your option)
-#  any later version.
+#  terms of the GNU General Public License as published by the Free Software
+#  Foundation, either version 3 of the License, or (at your option) any later
+#  version.
 #
 #  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
 #  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
-#  more details.
+#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+#  details.
 #
-#  You should have received a copy of the GNU Affero General Public License
-#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
+#  You should have received a copy of the GNU General Public License along with
+#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
 #
 ################################################################################
 """
@@ -44,7 +44,7 @@ def install_pip():
     with settings(warn_only=True):
         result = sudo('which pip')
     if result.failed:
-        apt.install('python-setuptools')
+        apt.install('python-pkg-resources', 'python-setuptools')
         sudo('easy_install pip')
         sudo('apt-get --assume-yes purge python-setuptools')
     pip('setuptools')
@@ -92,13 +92,14 @@ def configure_virtualenvwrapper(user, workon_home='/srv/envs'):
     update('.bashrc')
 
 
-def mkvirtualenv(name, user=None, upgrade_pip=True):
+def mkvirtualenv(name, python=None, user=None, upgrade_pip=True):
     """
     Make a new Python virtual environment.
     """
-    sudo('mkvirtualenv {0}'.format(name))
+    sudo('mkvirtualenv {} {}'.format('--python={}'.format(python) if python else '', name))
     if upgrade_pip:
         with workon(name):
+            pip('six')
             pip('pip', 'setuptools', 'wheel', 'ndg-httpsclient')
     if user:
         with cdvirtualenv(name):
