@@ -2,22 +2,22 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2016 Lance Edgar
+#  Copyright © 2010-2017 Lance Edgar
 #
 #  This file is part of Rattail.
 #
 #  Rattail is free software: you can redistribute it and/or modify it under the
-#  terms of the GNU Affero General Public License as published by the Free
-#  Software Foundation, either version 3 of the License, or (at your option)
-#  any later version.
+#  terms of the GNU General Public License as published by the Free Software
+#  Foundation, either version 3 of the License, or (at your option) any later
+#  version.
 #
 #  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
 #  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
-#  more details.
+#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+#  details.
 #
-#  You should have received a copy of the GNU Affero General Public License
-#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
+#  You should have received a copy of the GNU General Public License along with
+#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
 #
 ################################################################################
 """
@@ -26,29 +26,63 @@ Fabric Library for SSH
 
 from __future__ import unicode_literals, absolute_import
 
-from fabric.api import sudo, cd
-from fabric.contrib.files import exists, sed
+import warnings
 
-from rattail.fablib import mkdir
+from fabric.api import sudo, cd, settings
+from fabric.contrib.files import exists, sed, append
+
+from rattail.fablib import mkdir, agent_sudo
 from rattail.fablib.python import cdvirtualenv
 
 
-def cache_host_key(host, for_user='root'):
+def cache_host_key(host, for_user='root', with_agent=False, warn_only=True, identity=''):
     """
     Cache the SSH host key for the given host, for the given user.
     """
     user = None if for_user == 'root' else for_user
-    sudo('ssh -o StrictHostKeyChecking=no {} echo'.format(host), user=user)
+    _sudo = agent_sudo if with_agent else sudo
+    if identity:
+        identity = '-i {}'.format(identity)
+    cmd = 'ssh {} -o StrictHostKeyChecking=no {} echo'.format(identity, host)
+    if warn_only:
+        with settings(warn_only=True):
+            _sudo(cmd, user=user)
+    else:
+        _sudo(cmd, user=user)
+
+
+def uncache_host_key(host, for_user='root'):
+    """
+    Remove the cached SSH host key for the given host, for the given user.
+    """
+    user = None if for_user == 'root' else for_user
+    sudo('ssh-keygen -R {}'.format(host), user=user)
+
+
+def restart():
+    """
+    Restart the OpenSSH service
+    """
+    sudo('service ssh restart')
+
+
+def configure(allow_root=False):
+    """
+    Configure the OpenSSH service
+    """
+    path = '/etc/ssh/sshd_config'
+    value = 'without-password' if allow_root else 'no'
+    sed(path, r'^PermitRootLogin\s+.*', 'PermitRootLogin {}'.format(value), use_sudo=True)
+    entry = 'PasswordAuthentication no'
+    sed(path, r'^PasswordAuthentication\s+.*', entry, use_sudo=True)
+    append(path, entry, use_sudo=True)
+    restart()
 
 
 def configure_ssh(restrict_root=True):
-    """
-    Configure the OpenSSH service.
-    """
-    if restrict_root:
-        sed('/etc/ssh/sshd_config', r'^PermitRootLogin yes$', r'PermitRootLogin no', use_sudo=True)
-    sed('/etc/ssh/sshd_config', r'^#PasswordAuthentication yes$', r'PasswordAuthentication no', use_sudo=True)
-    sudo('service ssh restart')
+    warnings.warn("Function `ssh.configure_ssh()` is deprecated, please "
+                  "use `ssh.configure()` instead.", DeprecationWarning)
+    return configure(allow_root=not restrict_root)
 
 
 def establish_identity(envname, comment, user='rattail', home='/var/lib/rattail'):
