@@ -1,23 +1,23 @@
-# -*- coding: utf-8 -*-
+# -*- coding: utf-8; -*-
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2016 Lance Edgar
+#  Copyright © 2010-2017 Lance Edgar
 #
 #  This file is part of Rattail.
 #
 #  Rattail is free software: you can redistribute it and/or modify it under the
-#  terms of the GNU Affero General Public License as published by the Free
-#  Software Foundation, either version 3 of the License, or (at your option)
-#  any later version.
+#  terms of the GNU General Public License as published by the Free Software
+#  Foundation, either version 3 of the License, or (at your option) any later
+#  version.
 #
 #  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
 #  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
-#  more details.
+#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+#  details.
 #
-#  You should have received a copy of the GNU Affero General Public License
-#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
+#  You should have received a copy of the GNU General Public License along with
+#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
 #
 ################################################################################
 """
@@ -31,11 +31,17 @@ import logging
 from rattail.db import cache
 from rattail.db.util import QuerySequence
 from rattail.time import make_utc
+from rattail.util import progress_loop, OrderedDict
 
 
 log = logging.getLogger(__name__)
 
 
+# TODO
+class ImportLimitReached(Exception):
+    pass
+
+
 class Importer(object):
     """
     Base class for all data importers.
@@ -78,6 +84,8 @@ class Importer(object):
 
     def __init__(self, config=None, fields=None, key=None, **kwargs):
         self.config = config
+        self.enum = config.get_enum() if config else None
+        self.model_class = kwargs.pop('model_class', self.get_model_class())
         self.fields = fields or self.supported_fields
         if key is not None:
             self.key = key
@@ -88,12 +96,14 @@ class Importer(object):
                 if field not in self.fields:
                     raise ValueError("Key field '{}' must be included in effective fields "
                                      "for {}".format(field, self.__class__.__name__))
-        self.model_class = kwargs.pop('model_class', self.model_class)
         self.model_name = kwargs.pop('model_name', self.model_name)
         if not self.model_name and self.model_class:
             self.model_name = self.model_class.__name__
         self._setup(**kwargs)
 
+    def get_model_class(self):
+        return self.model_class
+
     def fields_active(self, fields):
         """
         Convenience method to check if any of the given fields are currently
@@ -121,6 +131,10 @@ class Importer(object):
         Perform any cleanup after import, if necessary.
         """
 
+    def progress_loop(self, func, items, factory=None, **kwargs):
+        factory = factory or self.progress
+        return progress_loop(func, items, factory, **kwargs)
+
     def import_data(self, host_data=None, now=None, **kwargs):
         """
         Import some data!  This is the core body of logic for that, regardless
@@ -131,7 +145,9 @@ class Importer(object):
         if kwargs:
             self._setup(**kwargs)
         self.setup()
-        created = updated = deleted = []
+        created = []
+        updated = []
+        deleted = []
 
         # Get complete set of normalized host data.
         if host_data is None:
@@ -140,16 +156,15 @@ class Importer(object):
         # Prune duplicate keys from host/source data.  This is for the sake of
         # sanity since duplicates typically lead to a ping-pong effect, where a
         # "clean" (change-less) import is impossible.
-        unique = {}
+        unique = OrderedDict()
         for data in host_data:
             key = self.get_key(data)
             if key in unique:
                 log.warning("duplicate records detected from {} for key: {}".format(
                     self.host_system_title, key))
-            unique[key] = data
-        host_data = []
-        for key in sorted(unique):
-            host_data.append(unique[key])
+            else:
+                unique[key] = data
+        host_data = list(unique.itervalues())
 
         # Cache local data if appropriate.
         if self.caches_local_data:
@@ -170,21 +185,20 @@ class Importer(object):
         self.teardown()
         return created, updated, deleted
 
-    def _import_create_update(self, data):
+    def _import_create_update(self, data, created=None, updated=None):
         """
         Import the given data; create and/or update records as needed.
         """
-        created, updated = [], []
+        if created is None:
+            created = []
+        if updated is None:
+            updated = []
         count = len(data)
         if not count:
             return created, updated
 
-        prog = None
-        if self.progress:
-            prog = self.progress("Importing {} data".format(self.model_name), count)
-
-        for i, host_data in enumerate(data, 1):
-
+        def import_(host_data, i):
+            
             # Fetch local object, using key from host data.
             key = self.get_key(host_data)
             local_object = self.get_local_object(key)
@@ -200,10 +214,10 @@ class Importer(object):
                     updated.append((local_object, local_data, host_data))
                     if self.max_update and len(updated) >= self.max_update:
                         log.warning("max of {} *updated* records has been reached; stopping now".format(self.max_update))
-                        break
+                        raise ImportLimitReached()
                     if self.max_total and (len(created) + len(updated)) >= self.max_total:
                         log.warning("max of {} *total changes* has been reached; stopping now".format(self.max_total))
-                        break
+                        raise ImportLimitReached()
 
             # If we did not yet have a local object, create it using host data.
             elif not local_object and self.create:
@@ -215,10 +229,10 @@ class Importer(object):
                         self.cached_local_data[key] = {'object': local_object, 'data': self.normalize_local_object(local_object)}
                     if self.max_create and len(created) >= self.max_create:
                         log.warning("max of {} *created* records has been reached; stopping now".format(self.max_create))
-                        break
+                        raise ImportLimitReached()
                     if self.max_total and (len(created) + len(updated)) >= self.max_total:
                         log.warning("max of {} *total changes* has been reached; stopping now".format(self.max_total))
-                        break
+                        raise ImportLimitReached()
                 else:
                     log.debug("did NOT create new {} for key: {}".format(self.model_name, key))
 
@@ -226,11 +240,10 @@ class Importer(object):
             if not self.batch_size or (len(created) + len(updated)) % self.batch_size == 0:
                 self.flush_create_update()
 
-            if prog:
-                prog.update(i)
-        if prog:
-            prog.destroy()
-
+        try:
+            self.progress_loop(import_, data, message="Importing {} data".format(self.model_name))
+        except ImportLimitReached:
+            pass
         self.flush_create_update_final()
         return created, updated
 
@@ -254,30 +267,21 @@ class Importer(object):
         deleting = self.get_deletion_keys() - host_keys
         count = len(deleting)
         log.debug("found {} instances to delete".format(count))
-        if count:
 
-            prog = None
-            if self.progress:
-                prog = self.progress("Deleting {} data".format(self.model_name), count)
+        def delete(key, i):
+            cached = self.cached_local_data.pop(key)
+            obj = cached['object']
+            if self.delete_object(obj):
+                deleted.append((obj, cached['data']))
+                if self.max_delete and len(deleted) >= self.max_delete:
+                    log.warning("max of {} *deleted* records has been reached; stopping now".format(self.max_delete))
+                    raise ImportLimitReached()
+                if self.max_total and (changes + len(deleted)) >= self.max_total:
+                    log.warning("max of {} *total changes* has been reached; stopping now".format(self.max_total))
+                    raise ImportLimitReached()
 
-            for i, key in enumerate(sorted(deleting), 1):
-
-                cached = self.cached_local_data.pop(key)
-                obj = cached['object']
-                if self.delete_object(obj):
-                    deleted.append((obj, cached['data']))
-
-                    if self.max_delete and len(deleted) >= self.max_delete:
-                        log.warning("max of {} *deleted* records has been reached; stopping now".format(self.max_delete))
-                        break
-                    if self.max_total and (changes + len(deleted)) >= self.max_total:
-                        log.warning("max of {} *total changes* has been reached; stopping now".format(self.max_total))
-                        break
-
-                if prog:
-                    prog.update(i)
-            if prog:
-                prog.destroy()
+        self.progress_loop(delete, sorted(deleting),
+                           message="Deleting {} data".format(self.model_name))
 
         return deleted
 
@@ -348,13 +352,15 @@ class Importer(object):
         """
         return tuple(normal['data'].get(k) for k in self.key)
 
-    def normalize_cache_object(self, obj):
+    def normalize_cache_object(self, obj, data=None):
         """
         Normalizer for cached local data.  This returns a simple dict with
         ``'object'`` and ``'data'`` keys; values are the raw object and its
         normalized data dict, respectively.
         """
-        return {'object': obj, 'data': self.normalize_local_object(obj)}
+        if data is None:
+            data = self.normalize_local_object(obj)
+        return {'object': obj, 'data': data}
 
     def normalize_local_object(self, obj):
         """
@@ -435,18 +441,19 @@ class Importer(object):
         from the given host data.  This may return ``None`` if no object is
         created.
         """
-        obj = self.new_object(key)
-        if obj:
-            return self.update_object(obj, host_data)
+        if not host_data.get('_deleted_', False):
+            obj = self.new_object(key)
+            if obj:
+                return self.update_object(obj, host_data)
 
-    def update_object(self, obj, host_data, local_data=None):
+    def update_object(self, obj, host_data, local_data=None, all_fields=False):
         """
         Update the local data object with the given host data, and return the
         object.
         """
         for field in self.simple_fields:
-            if field not in self.key and field in self.fields:
-                if not local_data or local_data[field] != host_data[field]:
+            if field not in self.key and field in host_data and (all_fields or field in self.fields):
+                if not local_data or field not in local_data or local_data[field] != host_data[field]:
                     setattr(obj, field, host_data[field])
         return obj
 
@@ -494,7 +501,7 @@ class FromQuery(Importer):
         """
         raise NotImplementedError
 
-    def get_host_objects(self, progress=None):
+    def get_host_objects(self):
         """
         Returns (raw) query results as a sequence.
         """
@@ -510,6 +517,67 @@ class FromDjango(FromQuery):
         return self.host_model_class.objects.all()
 
 
+class BatchImporter(Importer):
+    """
+    Base class for importers which split their jobs up into batches
+    """
+
+    def import_data(self, host_data=None, now=None, **kwargs):
+        """
+        Import some data!  We must override the default logic here, in order to
+        chop up the job into proper batches.  This is because image data is
+        relatively large, and fetching all at once is not performant.
+        """
+        if host_data is not None:
+            raise ValueError("User-provided host data is not supported")
+
+        self.now = now or make_utc(tzinfo=True)
+        if kwargs:
+            self._setup(**kwargs)
+        self.setup()
+        created = []
+        updated = []
+        deleted = []
+
+        self.host_index = 0
+        host_data = self.normalize_host_data()
+        while host_data:
+            if self.caches_local_data:
+                self.cached_local_data = self.cache_local_data(host_data)
+            created, updated = self._import_create_update(host_data, created=created, updated=updated)
+
+            if self.max_create and len(created) >= self.max_create:
+                log.warning("max of {} *created* records has been reached; stopping now".format(self.max_create))
+                break
+            if self.max_update and len(updated) >= self.max_update:
+                log.warning("max of {} *updated* records has been reached; stopping now".format(self.max_update))
+                break
+            if self.max_total and (len(created) + len(updated)) >= self.max_total:
+                log.warning("max of {} *total changes* has been reached; stopping now".format(self.max_total))
+                break
+
+            self.host_index += self.batch_size
+            host_data = self.normalize_host_data()
+
+        self.teardown()
+        return created, updated, deleted
+
+    def cache_local_data(self, host_data=None):
+        if host_data is None:
+            raise ValueError("Must provide host data to this method")
+        if len(self.key) != 1:
+            raise RuntimeError("Compound key {} not supported for batch importer: {}".format(self.key, self))
+        key = self.key[0]
+        keys = [data[key] for data in host_data]
+        if keys:
+            query = self.session.query(self.model_class)\
+                                .filter(getattr(self.model_class, key).in_(keys))
+            return self.cache_model(self.model_class, key=self.get_cache_key,
+                                    query=query, query_options=self.cache_query_options(),
+                                    normalizer=self.normalize_cache_object)
+        return {} # empty cache
+
+
 class BulkImporter(Importer):
     """
     Base class for bulk data importers.
