@@ -1,23 +1,23 @@
-# -*- coding: utf-8 -*-
+# -*- coding: utf-8; -*-
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2016 Lance Edgar
+#  Copyright © 2010-2017 Lance Edgar
 #
 #  This file is part of Rattail.
 #
 #  Rattail is free software: you can redistribute it and/or modify it under the
-#  terms of the GNU Affero General Public License as published by the Free
-#  Software Foundation, either version 3 of the License, or (at your option)
-#  any later version.
+#  terms of the GNU General Public License as published by the Free Software
+#  Foundation, either version 3 of the License, or (at your option) any later
+#  version.
 #
 #  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
 #  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
-#  more details.
+#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+#  details.
 #
-#  You should have received a copy of the GNU Affero General Public License
-#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
+#  You should have received a copy of the GNU General Public License along with
+#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
 #
 ################################################################################
 """
@@ -26,14 +26,16 @@ Rattail Model Importers
 
 from __future__ import unicode_literals, absolute_import
 
+import datetime
 import logging
 
 from sqlalchemy import orm
 from sqlalchemy.orm.exc import NoResultFound
 
-from rattail.db import model
-from rattail.importing import ToSQLAlchemy
-from rattail.db.util import format_phone_number, normalize_phone_number
+from rattail.db import model, auth
+from rattail.importing import ToSQLAlchemy, BatchImporter
+from rattail.db.util import maxlen, normalize_full_name, format_phone_number, normalize_phone_number
+from rattail.time import localtime, make_utc
 
 
 log = logging.getLogger(__name__)
@@ -44,6 +46,11 @@ class ToRattail(ToSQLAlchemy):
     Base class for all Rattail model importers.
     """
     key = 'uuid'
+    extension_fields = []
+
+    @property
+    def simple_fields(self):
+        return super(ToRattail, self).simple_fields + self.extension_fields
 
 
 class PersonImporter(ToRattail):
@@ -179,6 +186,26 @@ class UserImporter(ToRattail):
     """
     model_class = model.User
 
+    @property
+    def supported_fields(self):
+        return super(UserImporter, self).supported_fields + ['plain_password']
+
+    def normalize_local_object(self, user):
+        data = super(UserImporter, self).normalize_local_object(user)
+        if 'plain_password' in self.fields:
+            data['plain_password'] = None # can't decrypt stored password
+        return data
+
+    def update_object(self, user, data, local_data=None):
+        from rattail.db import auth
+
+        user = super(UserImporter, self).update_object(user, data, local_data)
+        if user:
+            if 'plain_password' in self.fields:
+                if data['plain_password']:
+                    auth.set_user_password(user, data['plain_password'])
+            return user
+
 
 class AdminUserImporter(UserImporter):
     """
@@ -200,7 +227,7 @@ class AdminUserImporter(UserImporter):
         return data
 
     def update_object(self, user, data, local_data=None):
-        user = super(UserImporter, self).update_object(user, data, local_data)
+        user = super(AdminUserImporter, self).update_object(user, data, local_data)
         if user:
             if 'admin' in self.fields:
                 admin = self.get_admin()
@@ -506,7 +533,8 @@ class CustomerImporter(ToRattail):
     Customer data importer.
     """
     model_class = model.Customer
-    name_fields = [
+    person_fields = [
+        'person_uuid',
         'first_name',
         'last_name',
     ]
@@ -514,6 +542,14 @@ class CustomerImporter(ToRattail):
         'phone_number',
         'phone_number_2',
     ]
+    address_fields = [
+        'address_street',
+        'address_street2',
+        'address_city',
+        'address_state',
+        'address_zipcode',
+        'address_invalid',
+    ]
     group_fields = [
         'group_id',
         'group_id_2',
@@ -521,7 +557,7 @@ class CustomerImporter(ToRattail):
 
     @property
     def supported_fields(self):
-        return self.simple_fields + self.name_fields + self.phone_fields + self.group_fields + [
+        return self.simple_fields + self.person_fields + self.phone_fields + self.address_fields + self.group_fields + [
             'email_address',
         ]
 
@@ -531,11 +567,13 @@ class CustomerImporter(ToRattail):
 
     def cache_query_options(self):
         options = []
-        if self.fields_active(self.name_fields):
+        if self.fields_active(self.person_fields):
             options.append(orm.joinedload(model.Customer._people)\
                            .joinedload(model.CustomerPerson.person))
         if self.fields_active(self.phone_fields):
             options.append(orm.joinedload(model.Customer.phones))
+        if self.fields_active(self.address_fields):
+            options.append(orm.joinedload(model.Customer.addresses))
         if 'email_address' in self.fields:
             options.append(orm.joinedload(model.Customer.email))
         if self.fields_active(self.group_fields):
@@ -545,8 +583,10 @@ class CustomerImporter(ToRattail):
 
     def normalize_local_object(self, customer):
         data = super(CustomerImporter, self).normalize_local_object(customer)
-        if self.fields_active(self.name_fields):
+        if self.fields_active(self.person_fields):
             person = customer.people[0] if customer.people else None
+            if 'person_uuid' in self.fields:
+                data['person_uuid'] = person.uuid if person else None
             if 'first_name' in self.fields:
                 data['first_name'] = person.first_name if person else None
             if 'last_name' in self.fields:
@@ -563,6 +603,15 @@ class CustomerImporter(ToRattail):
             email = customer.email
             data['email_address'] = email.address if email else None
 
+        if self.fields_active(self.address_fields):
+            address = customer.addresses[0] if customer.addresses else None
+            for field in self.address_fields:
+                if field in self.fields:
+                    if address:
+                        data[field] = getattr(address, field[8:])
+                    else:
+                        data[field] = None
+
         if 'group_id' in self.fields:
             group = customer.groups[0] if customer.groups else None
             data['group_id'] = group.id if group else None
@@ -583,14 +632,21 @@ class CustomerImporter(ToRattail):
     def update_object(self, customer, data, local_data=None):
         customer = super(CustomerImporter, self).update_object(customer, data, local_data)
 
-        if self.fields_active(self.name_fields):
+        if self.fields_active(self.person_fields):
+            new_person = False
             if not customer.people:
-                customer.people.append(model.Person())
+                new_person = True
+                if 'person_uuid' in self.fields:
+                    customer._people.append(model.CustomerPerson(person_uuid=data['person_uuid']))
+                else:
+                    customer.people.append(model.Person())
             person = customer.people[0]
             if 'first_name' in self.fields and person.first_name != data['first_name']:
                 person.first_name = data['first_name']
             if 'last_name' in self.fields and person.last_name != data['last_name']:
                 person.last_name = data['last_name']
+            if new_person and 'first_name' in self.fields and 'last_name' in self.fields:
+                person.display_name = normalize_full_name(person.first_name, person.last_name)
 
         if 'phone_number' in self.fields:
             phones = [phone for phone in customer.phones if phone.type == 'Voice']
@@ -632,6 +688,20 @@ class CustomerImporter(ToRattail):
                 if len(customer.emails):
                     del customer.emails[:]
 
+        if self.fields_active(self.address_fields):
+            if any([data[f] for f in self.address_fields if f in self.fields]):
+                if customer.addresses:
+                    address = customer.addresses[0]
+                else:
+                    address = model.CustomerMailingAddress()
+                    customer.addresses.append(address)
+                for field in self.address_fields:
+                    if field in self.fields:
+                        if getattr(address, field[8:]) != data[field]:
+                            setattr(address, field[8:], data[field])
+            elif customer.addresses:
+                customer.addresses.pop(0)
+
         if 'group_id' in self.fields:
             group_id = data['group_id']
             if group_id:
@@ -984,6 +1054,57 @@ class CategoryImporter(ToRattail):
     """
     model_class = model.Category
 
+    @property
+    def supported_fields(self):
+        return self.simple_fields + [
+            'department_number',
+        ]
+
+    def setup(self):
+        if 'department_number' in self.fields:
+            query = self.session.query(model.Department)\
+                                .filter(model.Department.number != None)
+            self.departments = self.cache_model(model.Department, key='number', query=query)
+
+    def cache_query_options(self):
+        options = []
+        if 'department_number' in self.fields:
+            options.append(orm.joinedload(model.Category.department))
+        return options
+
+    def normalize_local_object(self, category):
+        data = super(CategoryImporter, self).normalize_local_object(category)
+        if 'department_number' in self.fields:
+            data['department_number'] = category.department.number if category.department else None
+        return data
+
+    def get_department(self, number):
+        if hasattr(self, 'departments'):
+            return self.departments.get(number)
+        return self.session.query(model.Department)\
+                           .filter(model.Department.number == number)\
+                           .first()
+
+    def update_object(self, category, data, local_data=None):
+        category = super(CategoryImporter, self).update_object(category, data, local_data)
+
+        if 'department_number' in self.fields:
+            number = data['department_number']
+            if number:
+                dept = self.get_department(number)
+                if not dept:
+                    dept = model.Department()
+                    dept.number = number
+                    dept.name = "(created from import)"
+                    self.session.add(dept)
+                    if hasattr(self, 'departments'):
+                        self.departments[dept.number] = dept
+                category.department = dept
+            elif category.department:
+                category.department = None
+
+        return category
+
 
 class FamilyImporter(ToRattail):
     """
@@ -1050,14 +1171,20 @@ class ProductImporter(ToRattail):
         'vendor_id',
         'vendor_item_code',
         'vendor_case_cost',
+        'vendor_unit_cost',
     ]
 
+    extension_attr = None
+
+    maxlen_category_code = maxlen(model.Category.code)
+
     @property
     def supported_fields(self):
         return self.simple_fields + self.regular_price_fields + self.sale_price_fields + self.vendor_fields + [
             'brand_name',
             'department_number',
             'subdepartment_number',
+            'category_code',
             'category_number',
             'family_code',
             'report_code',
@@ -1074,7 +1201,11 @@ class ProductImporter(ToRattail):
             self.departments = self.cache_model(model.Department, key='number', query=query)
         if 'subdepartment_number' in self.fields:
             self.subdepartments = self.cache_model(model.Subdepartment, key='number')
-        if 'category_number' in self.fields:
+        if 'category_code' in self.fields:
+            query = self.session.query(model.Category)\
+                                .filter(model.Category.code != None)
+            self.categories = self.cache_model(model.Category, key='code', query=query)
+        elif 'category_number' in self.fields:
             self.categories = self.cache_model(model.Category, key='number')
         if 'family_code' in self.fields:
             self.families = self.cache_model(model.Family, key='code')
@@ -1095,7 +1226,7 @@ class ProductImporter(ToRattail):
             options.append(orm.joinedload(model.Product.department))
         if 'subdepartment_number' in self.fields:
             options.append(orm.joinedload(model.Product.subdepartment))
-        if 'category_number' in self.fields:
+        if 'category_code' in self.fields or 'category_number' in self.fields:
             options.append(orm.joinedload(model.Product.category))
         if 'family_code' in self.fields:
             options.append(orm.joinedload(model.Product.family))
@@ -1117,6 +1248,12 @@ class ProductImporter(ToRattail):
         if self.fields_active(self.vendor_fields):
             options.append(orm.joinedload(model.Product.cost))
             # options.append(orm.joinedload(model.Product.costs))
+
+        # eager join extension table if applicable
+        if self.extension_attr and self.fields_active(self.extension_fields):
+            cls, name = self.extension_attr
+            options.append(orm.joinedload(getattr(cls, name)))
+
         return options
 
     def normalize_local_object(self, product):
@@ -1128,6 +1265,8 @@ class ProductImporter(ToRattail):
             data['department_number'] = product.department.number if product.department else None
         if 'subdepartment_number' in self.fields:
             data['subdepartment_number'] = product.subdepartment.number if product.subdepartment else None
+        if 'category_code' in self.fields:
+            data['category_code'] = product.category.code if product.category else None
         if 'category_number' in self.fields:
             data['category_number'] = product.category.number if product.category else None
         if 'family_code' in self.fields:
@@ -1185,6 +1324,8 @@ class ProductImporter(ToRattail):
                 data['vendor_item_code'] = cost.code if cost else None
             if 'vendor_case_cost' in self.fields:
                 data['vendor_case_cost'] = cost.case_cost if cost else None
+            if 'vendor_unit_cost' in self.fields:
+                data['vendor_unit_cost'] = cost.unit_cost if cost else None
 
         return data
 
@@ -1209,7 +1350,14 @@ class ProductImporter(ToRattail):
                            .filter(model.Subdepartment.number == number)\
                            .first()
 
-    def get_category(self, number):
+    def get_category_by_code(self, code):
+        if hasattr(self, 'categories'):
+            return self.categories.get(code)
+        return self.session.query(model.Category)\
+                           .filter(model.Category.code == code)\
+                           .first()
+
+    def get_category_by_number(self, number):
         if hasattr(self, 'categories'):
             return self.categories.get(number)
         return self.session.query(model.Category)\
@@ -1291,10 +1439,33 @@ class ProductImporter(ToRattail):
             elif product.subdepartment:
                 product.subdepartment = None
 
-        if 'category_number' in self.fields:
+        if 'category_code' in self.fields:
+            code = data['category_code']
+            if code:
+                if len(code) > self.maxlen_category_code:
+                    log.warning("category code length ({}) exceeds max allowed ({}): {}".format(
+                        len(code), self.maxlen_category_code, code))
+                    code = code[:self.maxlen_category_code]
+
+                category = self.get_category_by_code(code)
+                if not category:
+                    category = model.Category()
+                    category.code = code
+                    try:
+                        category.number = int(code)
+                    except ValueError:
+                        pass
+                    self.session.add(category)
+                    if hasattr(self, 'categories'):
+                        self.categories[code] = category
+                product.category = category
+            elif product.category:
+                product.category = None
+
+        elif 'category_number' in self.fields:
             number = data['category_number']
             if number:
-                cat = self.get_category(number)
+                cat = self.get_category_by_number(number)
                 if not cat:
                     cat = model.Category()
                     cat.number = number
@@ -1358,7 +1529,7 @@ class ProductImporter(ToRattail):
                 if not tax:
                     tax = model.Tax()
                     tax.code = code
-                    tax.description = "(auto-created)"
+                    tax.description = code
                     tax.rate = 0
                     self.session.add(tax)
                     if hasattr(self, 'taxes'):
@@ -1472,9 +1643,26 @@ class ProductImporter(ToRattail):
                 else:
                     log.warning("product has no cost, so can't set vendor_case_cost: {}".format(product))
 
+        if 'vendor_unit_cost' in self.fields:
+            cost = data['vendor_unit_cost']
+            if data.get('vendor_id'):
+                if product.cost:
+                    product.cost.unit_cost = cost
+                else:
+                    log.warning("product has no cost, so can't set vendor_case_cost: {}".format(product))
+
         return product
 
 
+class ProductImageImporter(BatchImporter, ToRattail):
+    """
+    Importer for product images data.  Note that this uses the "batch" approach
+    because fetching all data up front is not performant when the host/local
+    systems are on different machines etc.
+    """
+    model_class = model.ProductImage
+
+
 class ProductCodeImporter(ToRattail):
     """
     Data importer for :class:`rattail.db.model.ProductCode`.
@@ -1538,6 +1726,9 @@ class ProductCodeImporter(ToRattail):
         if 'product_upc' in self.key:
             i = list(self.key).index('product_upc')
             product = self.get_product(key[i])
+            if not product:
+                log.error("product not found for key: {}".format(key))
+                return
             product._codes.append(code)
         return code
 
@@ -1670,7 +1861,7 @@ class ProductCostImporter(ToRattail):
                 self.session.flush()
                 if hasattr(self, 'vendors'):
                     self.vendors[vendor.id] = vendor
-            cost.vendor_uuid = vendor.uuid
+            cost.vendor = vendor
 
         if 'product_upc' in self.fields and 'product_uuid' not in self.fields:
             upc = data['product_upc']
@@ -1688,14 +1879,28 @@ class ProductCostImporter(ToRattail):
                 log.warning("duplicate products detected for UPC {}".format(upc.pretty()))
 
         if 'preferred' in self.fields:
-            if data['preferred'] and cost.preference != 1:
-                product = cost.product
-                product.costs.remove(cost)
-                product.costs.insert(0, cost)
-            elif data['preferred'] is False and cost.preference == 1:
-                product = cost.product
-                if len(product.costs) > 1:
-                    product.costs.remove(cost)
+            product = cost.product or self.session.query(model.Product).get(cost.product_uuid)
+            if data['preferred']:
+                if cost in product.costs:
+                    if cost.preference != 1:
+                        product.costs.remove(cost)
+                        product.costs.insert(0, cost)
+                elif product.costs:
+                    product.costs.insert(0, cost)
+                else:
+                    product.costs.append(cost)
+            else: # not preferred
+                if cost in product.costs:
+                    if cost.preference == 1:
+                        if len(product.costs) > 1:
+                            product.costs.remove(cost)
+                            product.costs.append(cost)
+                            product.costs.reorder()
+                        else:
+                            log.warning("cannot un-prefer cost, as product has only the one: {}".format(cost))
+                else:
+                    if not product.costs:
+                        log.warning("new cost will be preferred, as product has only the one: {}".format(self.get_key(data)))
                     product.costs.append(cost)
                     product.costs.reorder()
 
@@ -1707,3 +1912,52 @@ class ProductPriceImporter(ToRattail):
     Data importer for :class:`rattail.db.model.ProductPrice`.
     """
     model_class = model.ProductPrice
+
+
+class ProductStoreInfoImporter(ToRattail):
+    """
+    Data importer for :class:`rattail.db.model.ProductStoreInfo`.
+    """
+    model_class = model.ProductStoreInfo
+
+
+class CustomerOrderImporter(ToRattail):
+    """
+    Importer for CustomerOrder data
+    """
+    model_class = model.CustomerOrder
+
+
+class CustomerOrderItemImporter(ToRattail):
+    """
+    Importer for CustomerOrderItem data
+    """
+    model_class = model.CustomerOrderItem
+
+
+class CustomerOrderItemEventImporter(ToRattail):
+    """
+    Importer for CustomerOrderItemEvent data
+    """
+    model_class = model.CustomerOrderItemEvent
+
+    def setup(self):
+        super(CustomerOrderItemEventImporter, self).setup()
+
+        self.start_date = self.args.start_date
+        if self.start_date:
+            midnight = datetime.datetime.combine(self.start_date, datetime.time(0))
+            self.start_time = localtime(self.config, midnight)
+
+        self.end_date = self.args.end_date
+        if self.end_date:
+            midnight = datetime.datetime.combine(self.end_date + datetime.timedelta(days=1), datetime.time(0))
+            self.end_time = localtime(self.config, midnight)
+
+    def cache_query(self):
+        query = self.session.query(model.CustomerOrderItemEvent)
+        if self.start_date:
+            query = query.filter(model.CustomerOrderItemEvent.occurred >= make_utc(self.start_time))
+        if self.end_date:
+            query = query.filter(model.CustomerOrderItemEvent.occurred < make_utc(self.end_time))
+        return query
