@@ -2,22 +2,22 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2016 Lance Edgar
+#  Copyright © 2010-2017 Lance Edgar
 #
 #  This file is part of Rattail.
 #
 #  Rattail is free software: you can redistribute it and/or modify it under the
-#  terms of the GNU Affero General Public License as published by the Free
-#  Software Foundation, either version 3 of the License, or (at your option)
-#  any later version.
+#  terms of the GNU General Public License as published by the Free Software
+#  Foundation, either version 3 of the License, or (at your option) any later
+#  version.
 #
 #  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
 #  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
-#  more details.
+#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+#  details.
 #
-#  You should have received a copy of the GNU Affero General Public License
-#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
+#  You should have received a copy of the GNU General Public License along with
+#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
 #
 ################################################################################
 """
@@ -52,20 +52,16 @@ class ToRattailHandler(ToSQLAlchemyHandler):
     def make_session(self):
         return Session()
 
+    def begin_local_transaction(self):
+        self.session = self.make_session()
+        if hasattr(self, 'runas_username') and self.runas_username:
+            self.session.set_continuum_user(self.runas_username)
 
-class FromRattailToRattail(FromRattailHandler, ToRattailHandler):
+
+class FromRattailToRattailBase(object):
     """
-    Handler for Rattail -> Rattail data import.
+    Common base class for Rattail -> Rattail data import/export handlers.
     """
-    local_title = "Rattail (local)"
-    dbkey = 'host'
-
-    @property
-    def host_title(self):
-        return "Rattail ({})".format(self.dbkey)
-
-    def make_host_session(self):
-        return Session(bind=self.config.rattail_engines[self.dbkey])
 
     def get_importers(self):
         importers = OrderedDict()
@@ -108,15 +104,51 @@ class FromRattailToRattail(FromRattailHandler, ToRattailHandler):
         importers['ProductCode'] = ProductCodeImporter
         importers['ProductCost'] = ProductCostImporter
         importers['ProductPrice'] = ProductPriceImporter
+        importers['ProductStoreInfo'] = ProductStoreInfoImporter
+        importers['ProductImage'] = ProductImageImporter
         return importers
 
     def get_default_keys(self):
         keys = self.get_importer_keys()
         if 'AdminUser' in keys:
             keys.remove('AdminUser')
+        if 'ProductImage' in keys:
+            keys.remove('ProductImage')
         return keys
 
 
+class FromRattailToRattailImport(FromRattailToRattailBase, FromRattailHandler, ToRattailHandler):
+    """
+    Handler for Rattail -> Rattail data import.
+    """
+    local_title = "Rattail (local)"
+    dbkey = 'host'
+
+    @property
+    def host_title(self):
+        return "Rattail ({})".format(self.dbkey)
+
+    def make_host_session(self):
+        return Session(bind=self.config.rattail_engines[self.dbkey])
+
+# TODO: deprecate/remove this?
+FromRattailToRattail = FromRattailToRattailImport
+
+
+class FromRattailToRattailExport(FromRattailToRattailBase, FromRattailHandler, ToRattailHandler):
+    """
+    Handler for Rattail -> Rattail data import.
+    """
+    host_title = "Rattail (default)"
+
+    @property
+    def local_title(self):
+        return "Rattail ({})".format(self.dbkey)
+
+    def make_session(self):
+        return Session(bind=self.config.rattail_engines[self.dbkey])
+
+
 class FromRattail(FromSQLAlchemy):
     """
     Base class for Rattail -> Rattail data importers.
@@ -272,7 +304,16 @@ class BrandImporter(FromRattail, model.BrandImporter):
     pass
 
 class ProductImporter(FromRattail, model.ProductImporter):
-    pass
+
+    # TODO...
+    @property
+    def simple_fields(self):
+        fields = super(ProductImporter, self).simple_fields
+        fields.remove('unit_uuid')
+        fields.remove('regular_price_uuid')
+        fields.remove('current_price_uuid')
+        return fields
+
 
 class ProductCodeImporter(FromRattail, model.ProductCodeImporter):
     pass
@@ -282,3 +323,19 @@ class ProductCostImporter(FromRattail, model.ProductCostImporter):
 
 class ProductPriceImporter(FromRattail, model.ProductPriceImporter):
     pass
+
+class ProductStoreInfoImporter(FromRattail, model.ProductStoreInfoImporter):
+    pass
+
+
+class ProductImageImporter(FromRattail, model.ProductImageImporter):
+    """
+    Importer for product images.  Note that this uses the "batch" approach
+    because fetching all data up front is not performant when the host/local
+    systems are on different machines etc.
+    """
+
+    def query(self):
+        query = self.host_session.query(self.model_class)\
+                                 .order_by(self.model_class.uuid)
+        return query[self.host_index:self.host_index + self.batch_size]
