@@ -1,23 +1,23 @@
-# -*- coding: utf-8 -*-
+# -*- coding: utf-8; -*-
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2015 Lance Edgar
+#  Copyright © 2010-2017 Lance Edgar
 #
 #  This file is part of Rattail.
 #
 #  Rattail is free software: you can redistribute it and/or modify it under the
-#  terms of the GNU Affero General Public License as published by the Free
-#  Software Foundation, either version 3 of the License, or (at your option)
-#  any later version.
+#  terms of the GNU General Public License as published by the Free Software
+#  Foundation, either version 3 of the License, or (at your option) any later
+#  version.
 #
 #  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
 #  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
-#  more details.
+#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+#  details.
 #
-#  You should have received a copy of the GNU Affero General Public License
-#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
+#  You should have received a copy of the GNU General Public License along with
+#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
 #
 ################################################################################
 """
@@ -26,10 +26,13 @@ Email Framework
 
 from __future__ import unicode_literals, absolute_import
 
+import os
+import six
 import smtplib
 import logging
 from email.message import Message
 from email.mime.multipart import MIMEMultipart
+from email.mime.application import MIMEApplication
 from email.mime.text import MIMEText
 
 from mako.template import Template
@@ -53,7 +56,7 @@ def send_email(config, key, data={}, attachments=[], fallback_key=None):
     email = get_email(config, key, fallback_key)
     if email.get_enabled():
         msg = email.make_message(data, attachments)
-        deliver_message(config, msg)
+        deliver_message(config, key, msg)
     else:
         log.debug("skipping email of type '{0}' per config".format(key))
 
@@ -72,7 +75,7 @@ def iter_emails(config):
     """
     Iterate over all available email types.
     """
-    for module in config.getlist('rattail.mail', 'emails', default=[]):
+    for module in config.getlist('rattail.mail', 'emails', default=['rattail.emails']):
         module = import_module_path(module)
         for name in dir(module):
             obj = getattr(module, name)
@@ -81,7 +84,7 @@ def iter_emails(config):
                 yield obj
 
 
-def deliver_message(config, msg, recipients=UNSPECIFIED):
+def deliver_message(config, key, msg, recipients=UNSPECIFIED):
     """
     Deliver an email message using the given SMTP configuration.
     """
@@ -105,7 +108,12 @@ def deliver_message(config, msg, recipients=UNSPECIFIED):
     username = config.get('rattail.mail', 'smtp.username')
     password = config.get('rattail.mail', 'smtp.password')
 
-    if config.getbool('rattail.mail', 'send_emails', default=True):
+    if config.getbool('rattail.mail', 'send_feedback_only', usedb=False, default=False):
+        send = key == 'user_feedback'
+    else:
+        send = config.getbool('rattail.mail', 'send_emails', usedb=False, default=True)
+
+    if send:
 
         log.debug("connecting to server: {0}".format(server))
         session = smtplib.SMTP(server)
@@ -116,9 +124,10 @@ def deliver_message(config, msg, recipients=UNSPECIFIED):
         result = session.sendmail(msg['From'], recips, msg.as_string())
         log.debug("sendmail result is: {0}".format(repr(result)))
         session.quit()
+        return True
 
-    else:
-        log.debug("config says no emails, but would have sent one to: {0}".format(recips))
+    log.debug("config says no emails, but would have sent one to: {0}".format(recips))
+    return False
 
 
 class Email(object):
@@ -134,6 +143,7 @@ class Email(object):
 
     def __init__(self, config, key=None, fallback_key=None):
         self.config = config
+        self.enum = config.get_enum()
 
         if key:
             self.key = key
@@ -169,7 +179,7 @@ class Email(object):
         """
         Returns the value for the message's ``From:`` header.
 
-        :rtype: unicode
+        :rtype: str
         """
         sender = self.config.get('rattail.mail', '{0}.from'.format(self.key))
         if not sender:
@@ -207,42 +217,45 @@ class Email(object):
             recips = self.config.getlist('rattail.mail', 'default.{0}'.format(type_))
         return recips
 
-    def get_prefix(self, data={}):
+    def get_prefix(self, data={}, magic=True):
         """
         Returns a string to be used as the subject prefix for the message.
 
-        :rtype: unicode
+        :rtype: str
         """
         prefix = self.config.get('rattail.mail', '{0}.prefix'.format(self.key))
         if not prefix:
             prefix = self.config.get('rattail.mail', 'default.prefix')
-        return prefix or self.default_prefix
+        prefix = prefix or self.default_prefix
+        if magic and not self.config.production():
+            prefix = "[STAGE] {}".format(prefix)
+        return prefix
 
-    def get_subject(self, data={}):
+    def get_subject(self, data={}, render=True):
         """
         Returns the base value for the message's subject header, i.e. minus
         prefix.
 
-        :rtype: unicode
+        :rtype: str
         """
         subject = self.config.get('rattail.mail', '{0}.subject'.format(self.key),
                                   default=self.default_subject)
         if not subject:
             subject = self.config.get('rattail.mail', 'default.subject')
-        if subject:
+        if subject and render:
             subject = Template(subject).render(**data)
         return subject
 
-    def get_complete_subject(self, data={}):
+    def get_complete_subject(self, data={}, render=True):
         """
         Returns the value for the message's ``Subject:`` header, i.e. the base
         subject with the prefix applied.  Note that config may provide the
         complete subject also, in which case the prefix and base subject are
         not considered.
 
-        :rtype: unicode
+        :rtype: str
         """
-        return "{0} {1}".format(self.get_prefix(data), self.get_subject(data))
+        return "{} {}".format(self.get_prefix(data), self.get_subject(data, render=render))
 
     def get_template(self, type_):
         """
@@ -258,12 +271,37 @@ class Email(object):
                 except TopLevelLookupException:
                     pass
 
+    def normalize_attachments(self, attachments):
+        normalized = []
+        for attachment in attachments:
+            if isinstance(attachment, six.string_types):
+                attachment = self.normalize_attachment(attachment)
+            normalized.append(attachment)
+        return normalized
+
+    ATTACHMENT_MIME_MAP = {
+        '.pdf': 'pdf',
+        '.xls': 'vnd.ms-excel',
+    }
+
+    def normalize_attachment(self, path):
+        root, ext = os.path.splitext(path)
+        mimetype = self.ATTACHMENT_MIME_MAP.get(ext.lower())
+        if mimetype:
+            with open(path, 'rb') as f:
+                part = MIMEApplication(f.read(), mimetype)
+            filename = os.path.basename(path)
+            part.add_header('Content-Disposition', 'attachment; filename="{}"'.format(filename))
+            return part
+        raise ValueError("Magic is not (yet) supported, please construct your own attachments for file: {}".format(path))
+
     def make_message(self, data={}, attachments=[]):
         """
         Returns a proper email ``Message`` instance which may be sent via SMTP.
         """
         txt_template = self.get_template('txt')
         html_template = self.get_template('html')
+        attachments = self.normalize_attachments(attachments)
 
         if txt_template and html_template:
             msg = MIMEMultipart(_subtype='alternative', _subparts=[
