@@ -1,23 +1,23 @@
-# -*- coding: utf-8 -*-
+# -*- coding: utf-8; -*-
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2016 Lance Edgar
+#  Copyright © 2010-2017 Lance Edgar
 #
 #  This file is part of Rattail.
 #
 #  Rattail is free software: you can redistribute it and/or modify it under the
-#  terms of the GNU Affero General Public License as published by the Free
-#  Software Foundation, either version 3 of the License, or (at your option)
-#  any later version.
+#  terms of the GNU General Public License as published by the Free Software
+#  Foundation, either version 3 of the License, or (at your option) any later
+#  version.
 #
 #  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
 #  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
-#  more details.
+#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+#  details.
 #
-#  You should have received a copy of the GNU Affero General Public License
-#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
+#  You should have received a copy of the GNU General Public License along with
+#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
 #
 ################################################################################
 """
@@ -26,13 +26,14 @@ Time Utilities
 
 from __future__ import unicode_literals, absolute_import
 
+import calendar
 import datetime
 import warnings
 
 import pytz
 
 
-def localtime(config, time=None, key='default', tzinfo=True):
+def localtime(config, time=None, key='default', from_utc=False, tzinfo=True):
     """
     Return a datetime which has been localized to a particular timezone.  The
     :func:`timezone()` function will be used to obtain the timezone to which
@@ -57,6 +58,9 @@ def localtime(config, time=None, key='default', tzinfo=True):
         time = zone.normalize(time.astimezone(zone))
     elif time.tzinfo:
         time = zone.normalize(time.astimezone(zone))
+    elif from_utc:
+        time = pytz.utc.localize(time)
+        time = zone.normalize(time.astimezone(zone))
     else:
         time = zone.localize(time)
     if not tzinfo:
@@ -130,3 +134,20 @@ def get_sunday(date):
     if weekday == 6: # Sunday
         return date
     return date - datetime.timedelta(days=weekday + 1)
+
+
+def previous_month(date):
+    """
+    Returns a date object for the first day of "previous" month, where the
+    "current" month is determined by ``date`` param.
+    """
+    current = date.replace(day=1)
+    return (current - datetime.timedelta(days=1)).replace(day=1)
+
+
+def last_of_month(date):
+    """
+    Returns a date representing the last day of whichever month ``date`` falls in.
+    """
+    last_day = calendar.monthrange(date.year, date.month)[1]
+    return date.replace(day=last_day)
