@@ -1,23 +1,23 @@
-# -*- coding: utf-8 -*-
+# -*- coding: utf-8; -*-
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2016 Lance Edgar
+#  Copyright © 2010-2017 Lance Edgar
 #
 #  This file is part of Rattail.
 #
 #  Rattail is free software: you can redistribute it and/or modify it under the
-#  terms of the GNU Affero General Public License as published by the Free
-#  Software Foundation, either version 3 of the License, or (at your option)
-#  any later version.
+#  terms of the GNU General Public License as published by the Free Software
+#  Foundation, either version 3 of the License, or (at your option) any later
+#  version.
 #
 #  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
 #  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
-#  more details.
+#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+#  details.
 #
-#  You should have received a copy of the GNU Affero General Public License
-#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
+#  You should have received a copy of the GNU General Public License along with
+#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
 #
 ################################################################################
 """
@@ -27,6 +27,8 @@ Utilities
 from __future__ import unicode_literals, absolute_import
 
 import sys
+import datetime
+import decimal
 import subprocess
 
 from pkg_resources import iter_entry_points
@@ -37,6 +39,10 @@ except ImportError: # pragma no cover
     from ordereddict import OrderedDict
 
 
+# generic singleton to indicate an arg which isn't set etc.
+NOTSET = object()
+
+
 def capture_output(command):
     """
     Runs ``command`` and returns any output it produces.
@@ -127,6 +133,55 @@ def prettify(text):
     return ' '.join([x.capitalize() for x in words])
 
 
+def pretty_boolean(value):
+    """
+    Returns ``'Yes'`` or ``'No'`` or empty string if value is ``None``
+    """
+    if value is None:
+        return ""
+    return "Yes" if value else "No"
+
+
+def hours_as_decimal(hours):
+    """
+    Convert the given ``datetime.timedelta`` object into a Decimal whose
+    value is in terms of hours.
+    """
+    if hours is None:
+        return
+    minutes = (hours.days * 1440) + (hours.seconds / 60)
+    return decimal.Decimal('{:0.2f}'.format(minutes / 60.0))
+
+
+def pretty_hours(hours=None, seconds=None):
+    """
+    Format the given ``hours`` value (which is assumed to be a
+    ``datetime.timedelta`` object) as HH:MM.  Note that instead of providing
+    that, you can provide ``seconds`` instead and a delta will be generated.
+    """
+    if hours is None and seconds is None:
+        return ''
+    if hours is None:
+        hours = datetime.timedelta(seconds=seconds)
+    minutes = (hours.days * 1440) + (hours.seconds / 60)
+    return '{}:{:02d}'.format(minutes // 60, minutes % 60)
+
+
+def pretty_quantity(value, empty_zero=False):
+    """
+    Return a "pretty" version of the given value, as string.  This is meant primarily
+    for use with things like order quantities, so that e.g. 1.0000 => 1
+    """
+    if value is None:
+        return ''
+    if int(value) == value:
+        value = int(value)
+        if empty_zero and value == 0:
+            return ''
+        return str(value)
+    return str(value).rstrip('0')
+
+
 def progress_loop(func, items, factory, *args, **kwargs):
     """
     This will iterate over ``items`` and call ``func`` for each.  If a progress
@@ -135,8 +190,12 @@ def progress_loop(func, items, factory, *args, **kwargs):
     """
     message = kwargs.pop('message', None)
     count = kwargs.pop('count', None)
+    allow_cancel = kwargs.pop('allow_cancel', False)
     if count is None:
-        count = len(items)
+        try:
+            count = len(items)
+        except TypeError:
+            count = items.count()
     if not count:
         return True
 
@@ -152,4 +211,6 @@ def progress_loop(func, items, factory, *args, **kwargs):
             break
     if prog:
         prog.destroy()
+    if canceled and not allow_cancel:
+        raise RuntimeError("Operation was canceled")
     return not canceled
